@@ -1,17 +1,41 @@
+/********************************************************************\
+ * gtest-import-backend.cpp - Tests for import-backend              *
+ *                                                                  *
+ * Copyright (c) 2020 Christian Gruber                              *
+ *                                                                  *
+ * This program is free software; you can redistribute it and/or    *
+ * modify it under the terms of the GNU General Public License as   *
+ * published by the Free Software Foundation; either version 2 of   *
+ * the License, or (at your option) any later version.              *
+ *                                                                  *
+ * This program is distributed in the hope that it will be useful,  *
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
+ * GNU General Public License for more details.                     *
+ *                                                                  *
+ * You should have received a copy of the GNU General Public License*
+ * along with this program; if not, contact:                        *
+ *                                                                  *
+ * Free Software Foundation           Voice:  +1-617-542-5942       *
+ * 51 Franklin Street, Fifth Floor    Fax:    +1-617-542-2652       *
+ * Boston, MA  02110-1301,  USA       gnu@gnu.org                   *
+\********************************************************************/
+
 #include <gtk/gtk.h>
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcpp"
 #include <gtest/gtest.h>
+#pragma GCC diagnostic pop
+
 #include <gmock/gmock.h>
 
 #include <config.h>
 
 #include <gnc-datetime.hpp>
 
-extern "C"
-{
 #include <import-backend.h>
 #include <engine-helpers.h>
 #include <gnc-ui-util.h>
-}
 
 #include "gmock-gnc-prefs.h"
 #include "gmock-qofbook.h"
@@ -43,8 +67,9 @@ testing::Environment* const env = testing::AddGlobalTestEnvironment(new TestEnvi
 
 
 
-/* mock functions, which can not be mocked by mock classes */
+/* required fake functions from engine sources, which should not be linked to the test application */
 
+// fake function from qofutil.cpp
 gint
 safe_strcasecmp (const gchar * da, const gchar * db)
 {
@@ -52,6 +77,7 @@ safe_strcasecmp (const gchar * da, const gchar * db)
     return g_strcmp0(da, db);
 }
 
+// fake function from qoflog.cpp
 const char *
 qof_log_prettify (const char *name)
 {
@@ -59,7 +85,15 @@ qof_log_prettify (const char *name)
     return name;
 }
 
-// this is a slightly modified version of the function from engine-helpers.c
+// Fake qof_log_check suppresses logging.
+gboolean
+qof_log_check(QofLogModule log_module, QofLogLevel log_level)
+{
+    return FALSE;
+}
+
+// fake function from engine-helpers.c
+// this is a slightly modified version of the original function
 const char *
 gnc_get_num_action (const Transaction *trans, const Split *split)
 {
@@ -79,6 +113,21 @@ gnc_get_num_action (const Transaction *trans, const Split *split)
     else return NULL;
 }
 
+// fake function from gnc-commodity.c
+// this is a simplified version of the original function
+gboolean
+gnc_commodity_equiv(const gnc_commodity * a, const gnc_commodity * b)
+{
+    if (a == b) return TRUE;
+    if (!a || !b) return FALSE;
+
+    return TRUE;
+}
+
+
+/* required fake functions from app-utils sources, which should not be linked to the test application */
+
+// fake function from gnc-ui-util.c
 QofBook *
 gnc_get_current_book (void)
 {
@@ -114,8 +163,7 @@ class ImportBackendTest : public testing::Test
 protected:
     void SetUp()
     {
-        m_prefs = MockPrefsBackend::getInstance();
-        ASSERT_NE(m_prefs, nullptr);
+        gmock_gnc_prefs_set_backend(&m_prefs);
         m_import_acc = new MockAccount();
         m_dest_acc   = new MockAccount();
         m_trans      = new MockTransaction();
@@ -126,7 +174,7 @@ protected:
         using namespace testing;
 
         // define behaviour of m_import_acc
-        ON_CALL(*m_import_acc, getBook())
+        ON_CALL(*m_import_acc, get_book())
             .WillByDefault(Return(((TestEnvironment*)env)->m_book));
     }
 
@@ -135,10 +183,11 @@ protected:
         m_import_acc->free();
         m_dest_acc->free();
         m_trans->free();
+        g_list_free (m_splitList);
         m_split->free();
     }
 
-    MockPrefsBackend* m_prefs;
+    MockPrefsBackend  m_prefs;
     MockAccount*      m_import_acc;
     MockAccount*      m_dest_acc;
     MockTransaction*  m_trans;
@@ -153,35 +202,33 @@ protected:
 //! Test for function gnc_import_TransInfo_new()
 TEST_F(ImportBackendTest, CreateTransInfo)
 {
-    GncMockImportMatchMap imap(m_import_acc);
-    gchar* online_id;
 
     using namespace testing;
 
     //qof_instance_get (QOF_INSTANCE (split), "online-id", &online_id, NULL);
 
     // Define first split
-    ON_CALL(*m_trans, getSplit(0))
+    ON_CALL(*m_trans, get_split(0))
         .WillByDefault(Return(m_split));
-    ON_CALL(*m_trans, getSplitList())
+    ON_CALL(*m_trans, get_split_list())
         .WillByDefault(Return(m_splitList));
     // define description of the transaction
-    ON_CALL(*m_trans, getDescription())
+    ON_CALL(*m_trans, get_description())
         .WillByDefault(Return("This is the description"));
 
     // function gnc_import_TransInfo_new() should try to find account using the description from the transaction
-    EXPECT_CALL(imap, findAccount(_, StrEq("This is the description")))
+    EXPECT_CALL(*m_import_acc, find_account(_, StrEq("This is the description")))
         .WillOnce(Return(m_dest_acc));
 
     // call function to be tested
-    GNCImportTransInfo *trans_info = gnc_import_TransInfo_new(m_trans, &imap);
+    GNCImportTransInfo *trans_info = gnc_import_TransInfo_new(m_trans, m_import_acc);
 
     // check 'trans_info'
     EXPECT_EQ(gnc_import_TransInfo_get_fsplit(trans_info),  m_split);
     EXPECT_EQ(gnc_import_TransInfo_get_destacc(trans_info), m_dest_acc);
 
     // transaction is not open anymore
-    ON_CALL(*m_trans, isOpen())
+    ON_CALL(*m_trans, is_open())
         .WillByDefault(Return(false));
 
     // delete transaction info
@@ -201,7 +248,7 @@ protected:
         using namespace testing;
 
         // set bayesian import matching in preferences
-        ON_CALL(*m_prefs, getBool(StrEq(GNC_PREFS_GROUP_IMPORT), StrEq(GNC_PREF_USE_BAYES)))
+        ON_CALL(m_prefs, get_bool(StrEq(GNC_PREFS_GROUP_IMPORT), StrEq(GNC_PREF_USE_BAYES)))
             .WillByDefault(Return(true));
     }
 
@@ -220,7 +267,6 @@ TEST_F(ImportBackendBayesTest, CreateTransInfo)
 {
     using namespace testing;
 
-    GncMockImportMatchMap imap(m_import_acc);
     time64 date(GncDateTime(GncDate(2020, 3, 18)));
     struct tm *tm_struct;
     char local_day_of_week[16];
@@ -231,12 +277,12 @@ TEST_F(ImportBackendBayesTest, CreateTransInfo)
     gnc_tm_free(tm_struct);
 
     // Define first split
-    ON_CALL(*m_trans, getSplit(0))
+    ON_CALL(*m_trans, get_split(0))
         .WillByDefault(Return(m_split));
-    ON_CALL(*m_trans, getSplitList())
+    ON_CALL(*m_trans, get_split_list())
         .WillByDefault(Return(m_splitList));
     // Transaction has no further splits
-    ON_CALL(*m_trans, getSplit(Gt(0)))
+    ON_CALL(*m_trans, get_split(Gt(0)))
         .WillByDefault(Return(nullptr));
     // Define description and memo of first split
     // This transaction is used for testing tokenization of its content.
@@ -245,17 +291,17 @@ TEST_F(ImportBackendBayesTest, CreateTransInfo)
     //   * separators at the beginning and end of string
     //   * duplicated tokens within and between description text end memo
     // The token separator is space.
-    ON_CALL(*m_trans, getDescription())
+    ON_CALL(*m_trans, get_description())
         .WillByDefault(Return(" test  tokens within   description  tokens  "));
-    ON_CALL(*m_split, getMemo())
+    ON_CALL(*m_split, get_memo())
         .WillByDefault(Return("  test   the memo test "));
     // Define transaction date
-    ON_CALL(*m_trans, getDate())
+    ON_CALL(*m_trans, get_date())
         .WillByDefault(Return(date));
 
     // check tokens created from transaction
-    EXPECT_CALL(imap, findAccountBayes(AllOf(
-            Each(Not(IsEmpty())),                // tokens must not be empty strings
+    EXPECT_CALL(*m_import_acc, find_account_bayes(AllOf(
+            Each(Not(StrEq(""))),                // tokens must not be empty strings
             Each(Not(HasSubstr(" "))),           // tokens must not contain separator
             Not(HasDuplicates()),                // tokens must be unique
             Contains(StrEq(local_day_of_week)),  // tokens must contain local day of week
@@ -265,14 +311,14 @@ TEST_F(ImportBackendBayesTest, CreateTransInfo)
         .WillOnce(Return(m_dest_acc));
 
     // call function to be tested
-    GNCImportTransInfo *trans_info = gnc_import_TransInfo_new(m_trans, &imap);
+    GNCImportTransInfo *trans_info = gnc_import_TransInfo_new(m_trans, m_import_acc);
 
     // check 'trans_info'
     EXPECT_EQ(gnc_import_TransInfo_get_fsplit(trans_info),  m_split);
     EXPECT_EQ(gnc_import_TransInfo_get_destacc(trans_info), m_dest_acc);
 
     // transaction is not open anymore
-    ON_CALL(*m_trans, isOpen())
+    ON_CALL(*m_trans, is_open())
         .WillByDefault(Return(false));
 
     // delete transaction info
