@@ -23,10 +23,11 @@
 
 #include <config.h>
 
-#include "assoccell.h"
+#include "doclinkcell.h"
 #include "basiccell.h"
 #include "cell-factory.h"
 #include "combocell.h"
+#include "completioncell.h"
 #include "datecell.h"
 #include "formulacell.h"
 #include "numcell.h"
@@ -58,7 +59,7 @@ gnc_register_init (void)
 
     gnc_register_add_cell_type (RECN_CELL_TYPE_NAME, gnc_recn_cell_new);
 
-    gnc_register_add_cell_type (ASSOC_CELL_TYPE_NAME, gnc_assoc_cell_new);
+    gnc_register_add_cell_type (DOCLINK_CELL_TYPE_NAME, gnc_doclink_cell_new);
 
     gnc_register_add_cell_type (QUICKFILL_CELL_TYPE_NAME,
                                 gnc_quickfill_cell_new);
