@@ -47,6 +47,7 @@
 #include "gnucash-sheetP.h"
 #include "table-allgui.h"
 #include "Account.h"
+#include "gnc-glib-utils.h"
 
 #define GNC_PREF_AUTO_RAISE_LISTS "auto-raise-lists"
 
@@ -73,6 +74,7 @@ typedef struct _PopBox
     gunichar complete_char; /* char to be used for auto-completion */
 
     GList* ignore_strings;
+
 } PopBox;
 
 
@@ -324,8 +326,6 @@ gnc_combo_cell_destroy (BasicCell* bcell)
 
     if (box != NULL)
     {
-        GList* node;
-
         /* Don't destroy the qf if its not ours to destroy */
         if (FALSE == box->use_quickfill_cache)
         {
@@ -357,7 +357,7 @@ gnc_combo_cell_set_sort_enabled (ComboCell* cell, gboolean enabled)
         return;
 
     block_list_signals (cell);
-    gnc_item_list_set_sort_enabled (box->item_list, enabled);
+    gnc_item_list_set_sort_column (box->item_list, 0);
     unblock_list_signals (cell);
 }
 
@@ -536,7 +536,7 @@ gnc_combo_cell_type_ahead_search (const gchar* newval,
     GRegex* regex0 = g_regex_new (escaped_sep, 0, 0, NULL);
     char* rep_str = g_regex_replace_literal (regex0, escaped_newval, -1, 0,
                                              newval_rep, 0, NULL);
-    char* normal_rep_str = g_utf8_normalize (rep_str, -1, G_NORMALIZE_ALL);
+    char* normal_rep_str = g_utf8_normalize (rep_str, -1, G_NORMALIZE_NFC);
     GRegex *regex = g_regex_new (normal_rep_str, G_REGEX_CASELESS, 0, NULL);
 
     gboolean valid = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (full_store),
@@ -559,13 +559,27 @@ gnc_combo_cell_type_ahead_search (const gchar* newval,
     gtk_list_store_clear (box->tmp_store);
     unblock_list_signals (cell);
 
+    if (strlen (newval) == 0) {
+        /* Deleting everything in the cell shouldn't provide a search result for
+         * "" because that will just be the first MAX_NUM_MATCHES accounts which
+         * isn't very useful.
+         *
+         * Skip the search show the popup again with all accounts. Clear the
+         * temp store or the cell will be pre-filled with the first account.
+         */
+        gnc_item_list_set_temp_store (box->item_list, NULL);
+        gnc_item_edit_show_popup (box->item_edit);
+        box->list_popped = TRUE;
+        goto cleanup;
+    }
+
     while (valid && num_found < MAX_NUM_MATCHES)
     {
         gchar* str_data = NULL;
         gchar* normalized_str_data = NULL;
         gtk_tree_model_get (GTK_TREE_MODEL (full_store), &iter, 0,
                             &str_data, -1);
-        normalized_str_data = g_utf8_normalize (str_data, -1, G_NORMALIZE_ALL);
+        normalized_str_data = g_utf8_normalize (str_data, -1, G_NORMALIZE_NFC);
 
         if (g_regex_match (regex, normalized_str_data, 0, NULL))
         {
@@ -585,6 +599,8 @@ gnc_combo_cell_type_ahead_search (const gchar* newval,
         gnc_item_edit_show_popup (box->item_edit);
         box->list_popped = TRUE;
     }
+
+cleanup:
     g_regex_unref (regex);
     return match_str;
 }
@@ -638,6 +654,7 @@ gnc_combo_cell_modify_verify (BasicCell* _cell,
             *start_selection = *end_selection = *cursor_position;
             return;
         }
+
         match_str = quickfill_match (box->qf, newval);
 
         if (match_str != NULL) // Do we have a quickfill match
@@ -899,42 +916,41 @@ gnc_combo_cell_gui_move (BasicCell* bcell)
     gnc_item_edit_set_popup (box->item_edit, NULL, NULL,
                              NULL, NULL, NULL, NULL, NULL);
 
+    gnc_item_list_select (box->item_list, NULL);
+
     box->list_popped = FALSE;
 }
 
 static int
-popup_get_height (G_GNUC_UNUSED GtkWidget* widget,
+popup_get_height (GtkWidget* widget,
                   int space_available,
-                  int row_height,
+                  G_GNUC_UNUSED int row_height,
                   gpointer user_data)
 {
     PopBox* box = user_data;
     GtkScrolledWindow* scrollwin = GNC_ITEM_LIST(widget)->scrollwin;
-    GtkWidget *hsbar = gtk_scrolled_window_get_hscrollbar (scrollwin);
-    GtkStyleContext *context = gtk_widget_get_style_context (hsbar);
-    /* Note: gtk_scrolled_window_get_overlay_scrolling (scrollwin) always returns
-       TRUE so look for style class "overlay-indicator" on the scrollbar. */
-    gboolean overlay = gtk_style_context_has_class (context, "overlay-indicator");
-    int count, height;
+    int height;
 
-    count = gnc_item_list_num_entries (box->item_list);
-    height = count * (gnc_item_list_get_cell_height (box->item_list) + 2);
-
-    if (!overlay)
-    {
-        gint minh, nath;
-        gtk_widget_get_preferred_height (hsbar, &minh, &nath);
-        height = height + minh;
-    }
+    // if popup_allocation_height set use that
+    if (box->item_edit->popup_allocation_height != -1)
+        height = box->item_edit->popup_allocation_height;
+    else
+        height = gnc_item_list_get_popup_height (GNC_ITEM_LIST(widget));
 
     if (height < space_available)
     {
-        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
-                                        GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);
         // if the list is empty height would be 0 so return 1 instead to
         // satisfy the check_popup_height_is_true function
-        return height ? height : 1;
+        gint ret_height = height ? height : 1;
+
+        gtk_widget_set_size_request (GTK_WIDGET(scrollwin), -1, ret_height);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
+                                        GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);
+        return ret_height;
     }
+    else
+        gtk_widget_set_size_request (GTK_WIDGET(scrollwin), -1, -1);
+
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrollwin),
                                     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     return space_available;
@@ -1118,4 +1134,3 @@ gnc_combo_cell_set_autosize (ComboCell* cell, gboolean autosize)
 
     box->autosize = autosize;
 }
-
