@@ -29,6 +29,7 @@
 (use-modules (gnucash core-utils))
 (use-modules (gnucash json builder))            ;for building JSON options
 (use-modules (gnucash report html-utilities))
+(use-modules (gnucash report report-utilities))
 (use-modules (srfi srfi-9))
 (use-modules (ice-9 match))
 
@@ -106,8 +107,7 @@
       ((head . tail)
        (let ((pair (assq head nested-lst)))
          (cond
-          ((not pair) (list-cdr-set! nested-lst (1- (length nested-lst))
-                                     (path->nested-alist path newval)))
+          ((not pair) (set-cdr! (last-pair nested-lst) (path->nested-alist path newval)))
           ((null? tail) (set-cdr! pair newval))
           (else (loop (cdr pair) tail))))))))
 
@@ -211,6 +211,8 @@
                                      (cons 'point (list
                                                    (cons 'pointStyle #f)))))
                     (cons 'tooltips (list
+                                     (cons 'mode 'index)
+                                     (cons 'intersect #f)
                                      (cons 'callbacks (list
                                                        (cons 'label #f)))))
 
@@ -268,16 +270,17 @@
   (gnc:html-chart-get chart '(type)))
 
 (define (gnc:html-chart-set-type! chart type)
-  (gnc:html-chart-set! chart '(type) type))
+  (gnc:html-chart-set! chart '(type) type)
+  (gnc:html-chart-set! chart '(options tooltips intersect) (equal? type 'pie)))
 
 (define (gnc:html-chart-title chart)
   (gnc:html-chart-get chart '(options title text)))
 
 (define (gnc:html-chart-set-title! chart title)
-  (gnc:html-chart-set! chart '(options title text) title))
+  (gnc:html-chart-set! chart '(options title text) (list-to-vec title)))
 
 (define (gnc:html-chart-set-data-labels! chart labels)
-  (gnc:html-chart-set! chart '(data labels) labels))
+  (gnc:html-chart-set! chart '(data labels) (list-to-vec labels)))
 
 (define (gnc:html-chart-set-axes-display! chart display?)
   (gnc:html-chart-set! chart '(options scales xAxes (0) display) display?)
@@ -337,9 +340,8 @@
     (nested-alist-get options path)))
 
 (define (gnc:html-chart-set! chart path val)
-  (let ((options (gnc:html-chart-get-options-internal chart))
-        (val-vec (list-to-vec val)))
-    (nested-alist-set! options path val-vec)
+  (let ((options (gnc:html-chart-get-options-internal chart)))
+    (nested-alist-set! options path val)
     (gnc:html-chart-set-options-internal! chart options)))
 
 (define JS-Number-to-String "
@@ -351,8 +353,12 @@ var toLocaleStringSupportsOptions = (typeof Intl == 'object' && Intl && typeof I
 function numformat(amount) {
   if (toLocaleStringSupportsOptions) {
       return amount.toLocaleString(undefined, {style:formsty, currency:curriso});
-  } else {
+  } else if (formsty == 'percent') {
+      return (100 * amount).toLocaleString() + '%';
+  } else if (formsty == 'currency') {
       return currsym + amount.toLocaleString();
+  } else {
+      return amount.toLocaleString();
   }
 }
 ")
