@@ -22,7 +22,8 @@
 (define-module (gnucash report report-core))
 
 (eval-when (compile load eval expand)
-  (load-extension "libgnc-report" "scm_init_sw_report_module"))
+  (load-extension "libgnc-report" "scm_init_sw_report_module")
+  (load-extension "libgnc-expressions-guile" "scm_init_sw_expressions_module"))
 
 (use-modules (gnucash engine))
 (use-modules (gnucash utilities))
@@ -31,17 +32,22 @@
 (use-modules (gnucash gnome-utils))
 (use-modules (ice-9 match))
 (use-modules (srfi srfi-1))
+(use-modules (srfi srfi-2))
 (use-modules (srfi srfi-9))
+(use-modules (srfi srfi-26))
+(use-modules ((srfi srfi-69) #:prefix ht:))            ;for O(1) hash-table-size
 (use-modules (gnucash report report-register-hooks))
 (use-modules (gnucash report html-style-sheet))
 (use-modules (gnucash report html-document))
 (use-modules (gnucash report html-utilities))
 
-(load-and-reexport (sw_report))
+(load-and-reexport (sw_report)
+                   (sw_engine)
+                   (gnucash options))
 
-(export <report>)
 (export gnc:all-report-template-guids)
 (export gnc:custom-report-template-guids)
+(export gnc:custom-report-invoice-template-guids)
 (export gnc:define-report)
 (export gnc:delete-report)
 (export gnc:find-report-template)
@@ -70,14 +76,17 @@
 (export gnc:report-embedded-list)
 (export gnc:report-export-thunk)
 (export gnc:report-export-types)
+(export gnc:report-get-anchor)          ;return anchor by key
 (export gnc:report-id)
+(export gnc:report-is-invoice-report?)
 (export gnc:report-menu-name)
 (export gnc:report-name)
 (export gnc:report-needs-save?)
 (export gnc:report-options)
 (export gnc:report-render-html)
-(export gnc:report-run)
+(export gnc:render-report)
 (export gnc:report-serialize)
+(export gnc:report-add-anchor!)         ;add anchor, returns the integer key
 (export gnc:report-set-ctext!)
 (export gnc:report-set-dirty?!)
 (export gnc:report-set-editor-widget!)
@@ -111,7 +120,6 @@
 (export gnc:report-to-template-new)
 (export gnc:report-to-template-update)
 (export gnc:report-type)
-(export gnc:restore-report-by-guid)
 (export gnc:restore-report-by-guid-with-custom-template)
 
 ;; Terminology in this file:
@@ -166,7 +174,7 @@
   (make-new-record-template version name report-guid parent-type options-generator
                             options-cleanup-cb options-changed-cb
                             renderer in-menu? menu-path menu-name
-                            menu-tip export-types export-thunk)
+                            menu-tip hook export-types export-thunk)
   report-template?
   (version report-template-version)
   (report-guid report-template-report-guid report-template-set-report-guid!)
@@ -180,11 +188,12 @@
   (menu-path report-template-menu-path)
   (menu-name report-template-menu-name)
   (menu-tip report-template-menu-tip)
+  (hook report-template-hook)
   (export-types report-template-export-types)
   (export-thunk report-template-export-thunk))
 
 (define (make-report-template)
-  (make-new-record-template #f #f #f #f #f #f #f #f #t #f #f #f #f #f))
+  (make-new-record-template #f #f #f #f #f #f #f #f #t #f #f #f #f #f #f))
 (define gnc:report-template-version report-template-version)
 (define gnc:report-template-report-guid report-template-report-guid)
 (define gnc:report-template-set-report-guid! report-template-set-report-guid!)
@@ -200,6 +209,7 @@
 (define gnc:report-template-menu-path report-template-menu-path)
 (define gnc:report-template-menu-name report-template-menu-name)
 (define gnc:report-template-menu-tip report-template-menu-tip)
+(define gnc:report-template-hook report-template-hook)
 (define gnc:report-template-export-types report-template-export-types)
 (define gnc:report-template-export-thunk report-template-export-thunk)
 
@@ -208,16 +218,15 @@
   (G_ "One of your reports has a report-guid that is a duplicate. Please check the report system, especially your saved reports, for a report with this report-guid: "))
 (define rpterr-guid1 (G_ "Wrong report definition: "))
 (define rpterr-guid2 (G_ " Report is missing a GUID."))
-(define rptwarn-legacy
-  (G_ "Some reports stored in a legacy format were found. This format is not supported anymore so these reports may not have been restored properly."))
+
 (define (gui-error str)
   (if (gnucash-ui-is-running)
       (gnc-error-dialog '() str)
-      (gnc:error "report-impl.scm error: " str)))
+      (gnc:error "report-core.scm error: " str)))
 (define (gui-warning str)
   (if (gnucash-ui-is-running)
       (gnc-warning-dialog '() str)
-      (gnc:warn "report-impl.scm warning: " str)))
+      (gnc:warn "report-core.scm warning: " str)))
 (define (gui-error-missing-template template-name)
   (gui-error
    (string-append
@@ -236,6 +245,7 @@ not found.")))
   (define report-rec (make-report-template))
   (define allowable-fields (record-type-fields <report-template>))
   (define (not-a-field? fld) (not (memq fld allowable-fields)))
+  (define (xor . args) (fold (lambda (a b) (if a (if b #f a) b)) #f args))
 
   (let loop ((args args))
     (match args
@@ -251,6 +261,11 @@ not found.")))
           ((hash-ref *gnc:_report-templates_* report-guid)
            (gui-error (string-append rpterr-dupe report-guid)))
 
+          ;; has export-type but no export-thunk. or vice versa.
+          ((xor (gnc:report-template-export-thunk report-rec)
+                (gnc:report-template-export-types report-rec))
+           (gui-error (format #f "Export needs both thunk and types: ~a" report-guid)))
+
           ;; good: new report definition, store into report-templates hash
           (else
            (hash-set! *gnc:_report-templates_* report-guid report-rec)))))
@@ -279,43 +294,40 @@ not found.")))
          (gnc:report-template-renderer templ))))
 
 (define (gnc:report-template-new-options report-template)
-  (let ((generator (gnc:report-template-options-generator report-template))
-        (namer
-         (gnc:make-string-option
-          gnc:pagename-general gnc:optname-reportname "0a"
-          (N_ "Enter a descriptive name for this report.")
-          (G_ (gnc:report-template-name report-template))))
-        (stylesheet
-         (gnc:make-multichoice-option
-          gnc:pagename-general gnc:optname-stylesheet "0b"
-          (N_ "Select a stylesheet for the report.")
-          (string->symbol (N_ "Default"))
-          (map
-           (lambda (ss)
-             (vector
-              (string->symbol (gnc:html-style-sheet-name ss))
-              (gnc:html-style-sheet-name ss)
-              (string-append (gnc:html-style-sheet-name ss)
-                             " " (G_ "stylesheet."))))
-           (gnc:get-html-style-sheets)))))
-
-    (let ((options (if (procedure? generator)
+  (let* ((generator (gnc:report-template-options-generator report-template))
+         (options (if (procedure? generator)
                        (or (gnc:backtrace-if-exception generator)
                            (begin
                              (gnc:warn "BUG DETECTED: Scheme exception raised in "
                                        "report options generator procedure named "
                                        (procedure-name generator))
-                             (gnc:new-options)))
-                       (gnc:new-options))))
-      (or (gnc:lookup-option options gnc:pagename-general gnc:optname-reportname)
-          (gnc:register-option options namer))
-      (or (gnc:lookup-option options gnc:pagename-general gnc:optname-stylesheet)
-          (gnc:register-option options stylesheet))
-      options)))
+                             (gnc-new-optiondb)))
+                       (gnc-new-optiondb)))
+         (optiondb (gnc:optiondb options)))
+      (or
+       (gnc-lookup-option optiondb gnc:pagename-general gnc:optname-reportname)
+       (gnc-register-string-option optiondb
+        gnc:pagename-general gnc:optname-reportname "0a"
+        (N_ "Enter a descriptive name for this report.")
+        (G_ (gnc:report-template-name report-template))))
+      (or
+       (gnc-lookup-option optiondb gnc:pagename-general gnc:optname-stylesheet)
+       (gnc-register-multichoice-option
+        optiondb
+        gnc:pagename-general gnc:optname-stylesheet "0b"
+        (N_ "Select a stylesheet for the report.")
+        (N_ "Default")
+        (map
+         (lambda (ss)
+           (vector
+            (string->symbol (gnc:html-style-sheet-name ss))
+            (gnc:html-style-sheet-name ss)))
+         (gnc:get-html-style-sheets))))
+      options))
 
 ;; A <report> represents an instantiation of a particular report type.
 (define-record-type <report>
-  (make-report type id options dirty? needs-save? editor-widget ctext custom-template)
+  (make-report type id options dirty? needs-save? editor-widget ctext custom-template anchors)
   report?
   (type report-type report-set-type!)
   (id report-id report-set-id!)
@@ -324,6 +336,7 @@ not found.")))
   (needs-save? report-needs-save? report-set-needs-save?!)
   (editor-widget report-editor-widget report-set-editor-widget!)
   (ctext report-ctext report-set-ctext!)
+  (anchors report-get-anchors report-set-anchors!)
   (custom-template report-custom-template report-set-custom-template!))
 
 (define gnc:report-type report-type)
@@ -343,6 +356,18 @@ not found.")))
 (define gnc:report-custom-template report-custom-template)
 (define gnc:report-set-custom-template! report-set-custom-template!)
 
+;; add an anchor to the report instance list of anchors. return the
+;; integer key to the anchor.
+(define (gnc:report-add-anchor! report anchor)
+  (let* ((hash (report-get-anchors report))
+         (size (ht:hash-table-size hash)))
+    (ht:hash-table-set! hash size anchor)
+    size))
+
+;; retrieve the anchor from the report instance list of anchors.
+(define (gnc:report-get-anchor report id)
+  (ht:hash-table-ref (report-get-anchors report) id))
+
 (define (gnc:report-set-dirty?! report val)
   (gnc:report-set-dirty?-internal! report val)
   (let* ((template (hash-ref *gnc:_report-templates_* (gnc:report-type report)))
@@ -353,51 +378,32 @@ not found.")))
 ;; gnc:make-report instantiates a report from a report-template.
 ;; The actual report is stored away in a hash-table -- only the id is returned.
 (define (gnc:make-report template-id . rest)
-  (let* ((template-parent (gnc:report-template-parent-type
-                           (hash-ref *gnc:_report-templates_* template-id)))
+  (let* ((template (hash-ref *gnc:_report-templates_* template-id))
+         (template-parent (gnc:report-template-parent-type template))
          (report-type (or template-parent template-id))
          (custom-template (if template-parent template-id ""))
          (r (make-report
              report-type     ;; type
              #f              ;; id
-             #f              ;; options
+             (if (null? rest)
+                 (gnc:report-template-new-options template)
+                 (car rest)) ;; options
              #t              ;; dirty
              #f              ;; needs-save
              #f              ;; editor-widget
              #f              ;; ctext
              custom-template ;; custom-template
+             (ht:make-hash-table)
              ))
-         (template (hash-ref *gnc:_report-templates_* template-id)))
-    (let ((options (if (null? rest)
-                       (gnc:report-template-new-options template)
-                       (car rest))))
-      (gnc:report-set-options! r options)
-      (gnc:options-register-callback
-       #f #f
-       (lambda ()
-         (gnc:report-set-dirty?! r #t)
-         (let ((cb (gnc:report-template-options-changed-cb template)))
-           (if cb (cb r))))
-       options))
-    (gnc:report-set-id! r (gnc-report-add r))
-    (gnc:report-id r)))
-
-
-(define (gnc:restore-report-by-guid id template-id template-name options)
-  (if options
-      (let* ((r (make-report template-id id options #t #t #f #f ""))
-             (report-id (gnc-report-add r)))
-        (if (number? report-id)
-            (gnc:report-set-id! r report-id))
-        report-id)
-      (begin
-        (gui-error-missing-template template-name)
-        #f)))
+         (id (gnc-report-add r)))       ;returns an integer
+    (gnc:report-set-id! r id)
+    id))
 
 (define (gnc:restore-report-by-guid-with-custom-template
          id template-id template-name custom-template-id options)
   (if options
-      (let* ((r (make-report template-id id options #t #t #f #f custom-template-id))
+      (let* ((r (make-report template-id id options #t #t #f #f custom-template-id
+                             (ht:make-hash-table)))
              (report-id (gnc-report-add r)))
         (if (number? report-id)
             (gnc:report-set-id! r report-id))
@@ -435,27 +441,22 @@ not found.")))
              (gnc:report-name report)))))
 
 (define (gnc:report-name report)
-  (let* ((opt (gnc:report-options report)))
+  (let* ((opt (gnc:optiondb (gnc:report-options report))))
     (and opt
-         (gnc:option-value
-          (gnc:lookup-option opt gnc:pagename-general gnc:optname-reportname)))))
+         (gnc-optiondb-lookup-value opt gnc:pagename-general gnc:optname-reportname))))
 
 (define (gnc:report-stylesheet report)
   (gnc:html-style-sheet-find
-   (symbol->string (gnc:option-value
-                    (gnc:lookup-option
-                     (gnc:report-options report)
+   (symbol->string (gnc-optiondb-lookup-value
+                    (gnc:optiondb (gnc:report-options report))
                      gnc:pagename-general
-                     gnc:optname-stylesheet)))))
+                     gnc:optname-stylesheet))))
 
 (define (gnc:report-set-stylesheet! report stylesheet)
-  (gnc:option-set-value
-   (gnc:lookup-option
-    (gnc:report-options report)
-    gnc:pagename-general
-    gnc:optname-stylesheet)
-   (string->symbol
-    (gnc:html-style-sheet-name stylesheet))))
+  (gnc-set-option
+   (gnc:optiondb (gnc:report-options report))
+   gnc:pagename-general gnc:optname-stylesheet
+   (string->symbol (gnc:html-style-sheet-name stylesheet))))
 
 
 ;; Load and save helper functions
@@ -469,6 +470,21 @@ not found.")))
 (define (gnc:custom-report-template-guids)
   (map car (gnc:custom-report-templates-list)))
 
+(define (gnc:report-is-invoice-report? guid)
+  (let ((tmpl (gnc:find-report-template guid)))
+    (cond
+     ((not tmpl) #f)
+     ((eq? (gnc:report-template-hook tmpl) 'invoice) #t)
+     (else (and-let* ((type (gnc:report-template-parent-type tmpl))
+                      (template (gnc:find-report-template type)))
+             (eq? (gnc:report-template-hook template) 'invoice))))))
+
+;; return a list of the invoice report template guids.
+(define (gnc:custom-report-invoice-template-guids)
+  (hash-fold
+   (lambda (k v p) (if (gnc:report-is-invoice-report? k) (cons k p) p))
+    '() *gnc:_report-templates_*))
+
 (define (gnc:find-report-template guid)
   (hash-ref *gnc:_report-templates_* guid))
 
@@ -485,7 +501,7 @@ not found.")))
           (hash-map->list cons *gnc:_report-templates_*)))
 
 ;; This function should be called right before changing a custom-template's name
-;; to test if the new name is unique among the existting custom reports.
+;; to test if the new name is unique among the existing custom reports.
 ;; If not the calling function can prevent the name from being updated.
 (define (gnc:report-template-has-unique-name? templ-guid new-name)
   (or (not new-name)
@@ -530,24 +546,12 @@ not found.")))
     (gnc:report-template-name
      (hash-ref *gnc:_report-templates_* (gnc:report-type report))))
    (gnc:generate-restore-forms (gnc:report-options report) "options")
-   ;; 2.6->2.4 compatibility code prefix
-   ;; Temporary check to make the new report saving code more or less backwards
-   ;; compatible with older gnucash versions. This can be removed again in 2.8.
-   "(if (defined? 'gnc:restore-report-by-guid-with-custom-template)\n"
-   ;; end of 2.6->2.4 compatibility code prefix.
    (format
     #f "  (gnc:restore-report-by-guid-with-custom-template ~S ~S ~S ~S options)\n"
     (gnc:report-id report) (gnc:report-type report)
     (gnc:report-template-name
      (hash-ref *gnc:_report-templates_* (gnc:report-type report)))
     (gnc:report-custom-template report))
-   ;; 2.6->2.4 compatibility code suffix
-   (format
-    #f "  (gnc:restore-report-by-guid ~S ~S ~S options))\n"
-    (gnc:report-id report) (gnc:report-type report)
-    (gnc:report-template-name
-     (hash-ref *gnc:_report-templates_* (gnc:report-type report))))
-   ;; end of 2.6->2.4 compatibility code suffix.
    ")"))
 
 ;; Generate guile code required to recreate embedded report instances
@@ -592,8 +596,8 @@ not found.")))
              "      ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n"
              "      (let*\n"
              "        (\n"
-             "          (option (gnc:lookup-option options \"__general\" \"report-list\"))\n"
-             "          (saved-report-list (gnc:option-value option))\n"
+             "          (option (gnc-lookup-option (gnc:optiondb options) \"__general\" \"report-list\"))\n"
+             "          (saved-report-list (GncOption-get-value option))\n"
              "        )\n"
              "        (\n"
              "          (lambda (option)\n"
@@ -656,11 +660,14 @@ not found.")))
 (define (gnc:report-template-serialize report-template)
   (let* ((name (gnc:report-template-name report-template))
          (type (gnc:report-template-parent-type report-template))
-         (templ-name (gnc:report-template-name
-                      (hash-ref *gnc:_report-templates_* type)))
-         (options (gnc:report-template-new-options report-template))
-         (guid (gnc:report-template-report-guid report-template)))
-    (gnc:report-template-serialize-internal name type templ-name options guid)))
+         (tmpl (hash-ref *gnc:_report-templates_* type)))
+    (cond
+     ((not tmpl) (gnc:warn "gnc:report-template-serialize: cannot find template for " type) #f)
+     (else
+      (let ((templ-name (gnc:report-template-name tmpl))
+            (options (gnc:report-template-new-options report-template))
+            (guid (gnc:report-template-report-guid report-template)))
+        (gnc:report-template-serialize-internal name type templ-name options guid))))))
 
 ;; Convert a report into a report template and save this template in the savefile
 ;; Under specific conditions the we will attempt to replace the current report's
@@ -723,7 +730,7 @@ not found.")))
 
 (define (gnc:report-template-save-to-savefile report-template)
   (let ((saved-form (gnc:report-template-serialize report-template)))
-    (gnc-saved-reports-write-to-file saved-form #f)))
+    (and saved-form (gnc-saved-reports-write-to-file saved-form #f))))
 
 ;; save all custom reports, moving the old version of the
 ;; saved-reports file aside as a backup
@@ -753,6 +760,7 @@ not found.")))
         (and template
              (let* ((renderer (gnc:report-template-renderer template))
                     (stylesheet (gnc:report-stylesheet report))
+                    (_ (report-set-anchors! report (ht:make-hash-table)))
                     (doc (renderer report))
                     (html (cond
                            ((string? doc) doc)
@@ -763,18 +771,12 @@ not found.")))
                (gnc:report-set-dirty?! report #f)  ;; mark it clean
                html)))))
 
-;; looks up the report by id and renders it with gnc:report-render-html
-;; marks the cursor busy during rendering; returns the html
-(define (gnc:report-run id)
-  (let ((report (gnc-report-find id))
-        (html #f))
-    (gnc-set-busy-cursor '() #t)
-    (gnc:backtrace-if-exception
-     (lambda ()
-       (if report (set! html (gnc:report-render-html report #t)))))
-    (gnc-unset-busy-cursor '())
-    html))
-
+;; render report. will return a 2-element list: either (list html #f)
+;; where html is the report html string, or (list #f captured-error)
+;; where captured-error is the error string.
+(define (gnc:render-report report)
+  (define (get-report) (gnc:report-render-html report #t))
+  (gnc:apply-with-error-handling get-report '()))
 
 ;; "thunk" should take the report-type and the report template record
 (define (gnc:report-templates-for-each thunk)
@@ -785,9 +787,9 @@ not found.")))
 
 ;; return the list of reports embedded in the specified report
 (define (gnc:report-embedded-list options)
-  (let* ((option (gnc:lookup-option options "__general" "report-list")))
+  (let* ((option (gnc-lookup-option (gnc:optiondb options) "__general" "report-list")))
     (and option
-         (let ((opt-value (gnc:option-value option)))
+         (let ((opt-value (GncOption-get-value option)))
            (map car opt-value)))))
 
 ;; delete an existing report from the hash table and then call to
@@ -812,6 +814,17 @@ not found.")))
 ;; gnucash-cli helper and exported functions
 ;;
 
+(define (show-selected-reports pred? port)
+  (for-each
+   (lambda (template)
+     (format port "* ~a ~a\n"
+             (if (gnc:report-template-parent-type template) "C" " ")
+             (gnc:report-template-name template)))
+   (sort (hash-fold (lambda (k v p) (if (pred? v) (cons v p) p)) '()
+                    *gnc:_report-templates_*)
+         (lambda (a b) (gnc:string-locale<? (gnc:report-template-name a)
+                                            (gnc:report-template-name b))))))
+
 (define (stderr-log tmpl . args)
   (apply format (current-error-port) tmpl args)
   #f)
@@ -826,8 +839,10 @@ not found.")))
          (export-types (gnc:report-template-export-types template)))
 
     (cond
-     ((not export-thunk) (stderr-log "Report ~s has no export code\n" report))
-     ((not export-types) (stderr-log "Report ~s has no export-types\n" report))
+     ((not export-thunk)
+      (stderr-log "Only the following reports have export code:\n")
+      (show-selected-reports gnc:report-template-export-thunk (current-error-port))
+      (stderr-log "Use -R show to describe report\n"))
      ((not (assoc export-type export-types))
       (stderr-log "Export-type disallowed: ~a. Allowed types: ~a\n"
                   export-type (string-join (map car export-types) ", ")))
@@ -847,17 +862,7 @@ not found.")))
        '() *gnc:_report-templates_*)))
 
 (define-public (gnc:cmdline-report-list port)
-  (for-each
-   (lambda (template)
-     (format port "* ~a ~a\n"
-             (if (gnc:report-template-parent-type template) "C" " ")
-             (gnc:report-template-name template)))
-   (sort (hash-fold
-          (lambda (k v p) (if (gnc:report-template-in-menu? v) (cons v p) p))
-          '() *gnc:_report-templates_*)
-         (lambda (a b)
-           (gnc:string-locale<? (gnc:report-template-name a)
-                                (gnc:report-template-name b))))))
+  (show-selected-reports gnc:report-template-in-menu? port))
 
 (define-public (gnc:cmdline-report-show report port)
   (let ((templates (reportname->templates report)))
