@@ -47,13 +47,6 @@
   (split-balance-with-closing col-datum-get-split-balance-with-closing)
   (split-value-balance col-datum-get-split-value-balance))
 
-(define FOOTER-TEXT
-  (gnc:make-html-text
-   (G_ "WARNING: Foreign currency conversions, and unrealized gains
-calculations are not confirmed correct. This report may be modified
-without notice. Bug reports are very welcome at
-https://bugs.gnucash.org/")))
-
 ;; define all option's names and help text so that they are properly
 
 (define optname-startdate (N_ "Start Date"))
@@ -62,6 +55,9 @@ https://bugs.gnucash.org/")))
 (define optname-period (N_ "Period duration"))
 (define opthelp-period (N_ "Duration between time periods"))
 
+(define optname-reverse-chrono (N_ "Period order is most recent first"))
+(define opthelp-reverse-chrono (N_ "Period order is most recent first"))
+
 (define optname-dual-columns (N_ "Enable dual columns"))
 (define opthelp-dual-columns (N_ "Selecting this option will enable double-column \
 reporting."))
@@ -131,124 +127,79 @@ also show overall period profit & loss."))
 (define networth-barchart-uuid "cbba1696c8c24744848062c7f1cf4a72")
 (define pnl-barchart-uuid "80769921e87943adade887b9835a7685")
 
-(define periodlist
-  (list
-   (list 'disabled
-         (cons 'text (G_ "Disabled"))
-         (cons 'tip (G_ "Disabled")))
-
-   (list 'YearDelta
-         (cons 'text (G_ "Year"))
-         (cons 'tip (G_ "One Year.")))
-
-   (list 'HalfYearDelta
-         (cons 'text (G_ "Half Year"))
-         (cons 'tip (G_ "Half Year.")))
-
-   (list 'QuarterDelta
-         (cons 'text (G_ "Quarter"))
-         (cons 'tip (G_ "One Quarter.")))
-
-   (list 'MonthDelta
-         (cons 'text (G_ "Month"))
-         (cons 'tip (G_ "One Month.")))
-
-   (list 'TwoWeekDelta
-         (cons 'text (G_ "2Week"))
-         (cons 'tip (G_ "Two Weeks.")))
-
-   (list 'WeekDelta
-         (cons 'text (G_ "Week"))
-         (cons 'tip (G_ "One Week.")))))
-
-(define (keylist->vectorlist keylist)
-  (map
-   (lambda (item)
-     (vector
-      (car item)
-      (keylist-get-info keylist (car item) 'text)
-      (keylist-get-info keylist (car item) 'tip)))
-   keylist))
-
-(define (keylist-get-info keylist key info)
-  (assq-ref (assq-ref keylist key) info))
-
 ;; options generator
 (define (multicol-report-options-generator report-type)
-  (let* ((options (gnc:new-options))
-         (book (gnc-get-current-book))
-         (add-option
-          (lambda (new-option)
-            (gnc:register-option options new-option))))
+  (let* ((options (gnc-new-optiondb)))
 
     ;; date at which to report balance
     (gnc:options-add-date-interval!
      options gnc:pagename-general optname-startdate optname-enddate "c")
 
-    (add-option
-     (gnc:make-multichoice-callback-option
+    (gnc-register-multichoice-callback-option options
       gnc:pagename-general optname-period
       "c2" opthelp-period
-      'disabled
-      (keylist->vectorlist periodlist)
-      #f
+      "disabled"
+      (list
+       (vector 'disabled (G_ "Disabled"))
+       (vector 'YearDelta (G_ "One Year"))
+       (vector 'HalfYearDelta (G_ "Half Year"))
+       (vector 'QuarterDelta (G_ "Quarter Year"))
+       (vector 'MonthDelta (G_ "One Month"))
+       (vector 'TwoWeekDelta (G_ "Two Weeks"))
+       (vector 'WeekDelta (G_ "One Week")))
       (lambda (x)
         (let ((x (not (eq? x 'disabled))))
-          (gnc-option-db-set-option-selectable-by-name
+          (gnc-optiondb-set-option-selectable-by-name
            options
            gnc:pagename-general optname-disable-amount-indent
            (not x))
-          (gnc-option-db-set-option-selectable-by-name
+          (gnc-optiondb-set-option-selectable-by-name
            options
            gnc:pagename-general optname-dual-columns
            (not x))
+          (gnc-optiondb-set-option-selectable-by-name
+           options gnc:pagename-general optname-reverse-chrono x)
           (case report-type
             ((balsheet)
-             (gnc-option-db-set-option-selectable-by-name
+             (gnc-optiondb-set-option-selectable-by-name
               options gnc:pagename-general optname-include-chart x)
 
-             (gnc-option-db-set-option-selectable-by-name
+             (gnc-optiondb-set-option-selectable-by-name
               options gnc:pagename-general optname-startdate x))
 
             ((pnl)
-             (gnc-option-db-set-option-selectable-by-name
-              options gnc:pagename-general optname-include-overall-period x)))))))
+             (gnc-optiondb-set-option-selectable-by-name
+              options gnc:pagename-general optname-include-overall-period x))))))
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-disable-amount-indent
-      "c3" opthelp-disable-amount-indent #f))
+      "c3" opthelp-disable-amount-indent #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-include-chart
-      "c5" opthelp-include-chart #f))
+      "c5" opthelp-include-chart #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-general optname-dual-columns
-      "c4" opthelp-dual-columns #t))
+      "c4" opthelp-dual-columns #t)
 
-    (add-option
-     (gnc:make-multichoice-option
+    (gnc-register-simple-boolean-option options
+      gnc:pagename-general optname-reverse-chrono
+      "c5" opthelp-reverse-chrono #t)
+
+    (gnc-register-multichoice-option options
       gnc:pagename-general optname-options-summary
       "d" opthelp-options-summary
-      'never
-      (list (vector 'always
-                    (G_ "Always")
-                    (G_ "Always display summary."))
-            (vector 'never
-                    (G_ "Never")
-                    (G_ "Disable report summary.")))))
+      "never"
+      (list (vector 'always (G_ "Always"))
+            (vector 'never (G_ "Never"))))
 
     ;; accounts to work on
-    (add-option
-     (gnc:make-account-list-option
+    (gnc-register-account-list-option options
       gnc:pagename-accounts optname-accounts
       "a"
       opthelp-accounts
-      (lambda ()
-        (gnc:filter-accountlist-type
+      (gnc:filter-accountlist-type
          (list ACCT-TYPE-BANK ACCT-TYPE-CASH ACCT-TYPE-CREDIT
                ACCT-TYPE-ASSET ACCT-TYPE-LIABILITY
                ACCT-TYPE-STOCK ACCT-TYPE-MUTUAL ACCT-TYPE-CURRENCY
@@ -256,7 +207,6 @@ also show overall period profit & loss."))
                ACCT-TYPE-EQUITY ACCT-TYPE-INCOME ACCT-TYPE-EXPENSE
                ACCT-TYPE-TRADING)
          (gnc-account-get-descendants-sorted (gnc-get-current-root-account))))
-      #f #t))
 
     ;; the depth-limit option is not well debugged; it may be better
     ;; to disable it altogether
@@ -265,19 +215,18 @@ also show overall period profit & loss."))
      "b" opthelp-depth-limit 'all)
 
     ;; all about currencies
-    (add-option
-     (gnc:make-complex-boolean-option
+    (gnc-register-complex-boolean-option options
       pagename-commodities optname-common-currency
-      "b" opthelp-common-currency #f #f
+      "b" opthelp-common-currency #f
       (lambda (x)
         (for-each
          (lambda (optname)
-           (gnc-option-db-set-option-selectable-by-name
+           (gnc-optiondb-set-option-selectable-by-name
             options pagename-commodities optname x))
          (list optname-report-commodity
                optname-show-rates
                optname-show-foreign
-               optname-price-source)))))
+               optname-price-source))))
 
     (gnc:options-add-currency!
      options pagename-commodities
@@ -287,62 +236,51 @@ also show overall period profit & loss."))
      options pagename-commodities
      optname-price-source "d" 'pricedb-nearest)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-foreign
-      "e" opthelp-show-foreign #t))
+      "e" opthelp-show-foreign #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       pagename-commodities optname-show-rates
-      "f" opthelp-show-rates #t))
+      "f" opthelp-show-rates #t)
 
     ;; what to show for zero-balance accounts
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-show-zb-accts
-      "a" opthelp-show-zb-accts #t))
+      "a" opthelp-show-zb-accts #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-omit-zb-bals
-      "b" opthelp-omit-zb-bals #f))
+      "b" opthelp-omit-zb-bals #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-parent-balance-mode
-      "c" opthelp-parent-balance-mode #t))
+      "c" opthelp-parent-balance-mode #t)
 
     ;; some detailed formatting options
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-account-links
-      "e" opthelp-account-links #t))
+      "e" opthelp-account-links #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-amount-links
-      "e5" opthelp-amount-links #t))
+      "e5" opthelp-amount-links #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
+    (gnc-register-simple-boolean-option options
       gnc:pagename-display optname-account-full-name
-      "f" opthelp-account-full-name #f))
+      "f" opthelp-account-full-name #f)
 
-    (add-option
-     (gnc:make-simple-boolean-option
-      gnc:pagename-display optname-label-sections "g" opthelp-label-sections #t))
+    (gnc-register-simple-boolean-option options
+      gnc:pagename-display optname-label-sections "g" opthelp-label-sections #t)
 
-    (add-option
-     (gnc:make-simple-boolean-option
-      gnc:pagename-display optname-total-sections "h" opthelp-total-sections #t))
+    (gnc-register-simple-boolean-option options
+      gnc:pagename-display optname-total-sections "h" opthelp-total-sections #t)
 
     (when (eq? report-type 'pnl)
       ;; include overall period column?
-      (add-option
-       (gnc:make-simple-boolean-option
+      (gnc-register-simple-boolean-option options
         gnc:pagename-general optname-include-overall-period
-        "c6" opthelp-include-overall-period #f)))
+        "c6" opthelp-include-overall-period #f))
 
     (gnc:options-set-default-section options gnc:pagename-general)
 
@@ -350,6 +288,7 @@ also show overall period profit & loss."))
 
 (define* (add-multicolumn-acct-table
           table title accountlist maxindent get-cell-monetary-fn cols-data #:key
+          (reverse-cols? #f)
           (omit-zb-bals? #f)
           (show-zb-accts? #t)
           (disable-account-indent? #f)
@@ -411,28 +350,30 @@ also show overall period profit & loss."))
   (define (make-narrow-cell)
     (gnc:make-html-table-cell/min-width 1))
 
+  (define (show-depth? lvl)
+    (or (not depth-limit) (<= lvl depth-limit)))
+
   (define (add-indented-row indent label label-markup row-markup amount-indent rest)
-    (when (or (not depth-limit) (<= indent depth-limit))
-      (let* ((account-cell (if label-markup
-                               (gnc:make-html-table-cell/size/markup
-                                1 (if disable-account-indent? 1 (- maxindent indent))
-                                label-markup label)
-                               (gnc:make-html-table-cell/size
-                                1 (if disable-account-indent? 1 (- maxindent indent))
-                                label)))
-             (row (append
-                   (if disable-account-indent?
-                       '()
-                       (make-list-thunk indent make-narrow-cell))
-                   (list account-cell)
-                   (gnc:html-make-empty-cells
-                    (if amount-indenting? (1- amount-indent) 0))
-                   rest
-                   (gnc:html-make-empty-cells
-                    (if amount-indenting? (- maxindent amount-indent) 0)))))
-        (if row-markup
-            (gnc:html-table-append-row/markup! table row-markup row)
-            (gnc:html-table-append-row! table row)))))
+    (let* ((account-cell (if label-markup
+                             (gnc:make-html-table-cell/size/markup
+                              1 (if disable-account-indent? 1 (- maxindent indent))
+                              label-markup label)
+                             (gnc:make-html-table-cell/size
+                              1 (if disable-account-indent? 1 (- maxindent indent))
+                              label)))
+           (row (append
+                 (if disable-account-indent?
+                     '()
+                     (make-list-thunk indent make-narrow-cell))
+                 (list account-cell)
+                 (gnc:html-make-empty-cells
+                  (if amount-indenting? (1- amount-indent) 0))
+                 (if reverse-cols? (reverse rest) rest)
+                 (gnc:html-make-empty-cells
+                  (if amount-indenting? (- maxindent amount-indent) 0)))))
+      (if row-markup
+          (gnc:html-table-append-row/markup! table row-markup row)
+          (gnc:html-table-append-row! table row))))
 
   (define (monetary+ . monetaries)
     ;; usage: (monetary+ monetary...)
@@ -467,13 +408,13 @@ also show overall period profit & loss."))
            (lp rest
                (let ((converted (and show-orig-cur? convert-curr-fn
                                      (convert-curr-fn monetary col-datum))))
-                 (cons* (gnc:html-markup-br)
-                        (if anchor
+                 (cons* (if anchor
                             (gnc:html-markup-anchor anchor (or converted monetary))
                             (or converted monetary))
                         (if converted
                             (gnc:html-markup-i (gnc:html-markup "small" monetary " "))
                             "")
+                        (if (null? accum) "" (gnc:html-markup-br))
                         accum)))))))
 
   (define (account->depth acc)
@@ -547,8 +488,9 @@ also show overall period profit & loss."))
              ((_ . tail) (lp1 tail))))))))
 
   (define* (add-recursive-subtotal lvl lvl-acct #:key account-style-normal?)
-    (if (or show-zb-accts?
-            (is-not-zero? (account-and-descendants lvl-acct)))
+    (if (and (or show-zb-accts?
+                 (is-not-zero? (account-and-descendants lvl-acct)))
+             (show-depth? lvl))
         (add-indented-row lvl
                           (render-account lvl-acct (not account-style-normal?))
                           (if account-style-normal?
@@ -574,9 +516,10 @@ also show overall period profit & loss."))
   (define* (add-account-row lvl-curr curr #:key
                             (override-show-zb-accts? #f)
                             (account-indent 0))
-    (if (or show-zb-accts?
-            override-show-zb-accts?
-            (is-not-zero? (list curr)))
+    (if (and (or show-zb-accts?
+                 override-show-zb-accts?
+                 (is-not-zero? (list curr)))
+             (show-depth? lvl-curr))
         (add-indented-row lvl-curr
                           (render-account curr #f)
                           "text-cell"
@@ -676,9 +619,8 @@ also show overall period profit & loss."))
 
 (define (multicol-report-renderer report-obj report-type)
   (define (get-option pagename optname)
-    (gnc:option-value
-     (gnc:lookup-option
-      (gnc:report-options report-obj) pagename optname)))
+    (gnc-optiondb-lookup-value
+      (gnc:report-options report-obj) pagename optname))
 
   (gnc:report-starting (get-option gnc:pagename-general gnc:optname-reportname))
 
@@ -749,6 +691,11 @@ also show overall period profit & loss."))
            ((eq? report-type 'pnl) (list startdate enddate))
            (else (list enddate))))
 
+         (reverse-chrono? (get-option gnc:pagename-general optname-reverse-chrono))
+
+         (report-dates-vec (list->vector report-dates))
+         (num-report-dates (vector-length report-dates-vec))
+
          ;; an alist of (cons account account-cols-data) whereby
          ;; account-cols-data is a list of col-datum records
          (accounts-cols-data
@@ -777,23 +724,36 @@ also show overall period profit & loss."))
          ;; account-balances is a list of monetary amounts
          (accounts-balances
           (map
-           (lambda (acc)
-             (cons acc (let ((cols-data (assoc-ref accounts-cols-data acc)))
-                         (map col-datum-get-split-balance cols-data))))
-           accounts))
+           (match-lambda
+             ((acc . cols-data)
+              (cons acc (map col-datum-get-split-balance cols-data))))
+           accounts-cols-data))
 
          (accounts-balances-with-closing
           (map
-           (lambda (acc)
-             (cons acc (let ((cols-data (assoc-ref accounts-cols-data acc)))
-                         (map col-datum-get-split-balance-with-closing cols-data))))
-           accounts))
+           (match-lambda
+             ((acc . cols-data)
+              (cons acc (map col-datum-get-split-balance-with-closing cols-data))))
+           accounts-cols-data))
 
+         ;; generate an exchange-fn
          (exchange-fn (and common-currency
-                           (gnc:case-exchange-time-fn
-                            price-source common-currency
-                            (map xaccAccountGetCommodity accounts) enddate
-                            #f #f)))
+                           (gnc:case-exchange-time-fn price-source common-currency
+                                                      (gnc:accounts-get-commodities accounts #f)
+                                                      #f #f #f)))
+
+         ;; from col-idx, find effective date to retrieve pricedb
+         ;; entry or to limit transactions to calculate average-cost
+         ;; or weighted-average
+         (col-idx->price-date
+          (lambda (col-idx)
+            (cond
+             ((eq? price-source 'pricedb-latest) (current-time))
+             ((eq? col-idx 'overall-period) enddate)
+             ((eq? report-type 'balsheet) (vector-ref report-dates-vec col-idx))
+             ((= col-idx (- num-report-dates 2)) enddate)
+             ((eq? report-type 'pnl)
+              (decdate (vector-ref report-dates-vec (1+ col-idx)) DayDelta)))))
 
          ;; this function will convert the monetary found at col-idx
          ;; into report-currency if the latter exists. The price
@@ -806,13 +766,8 @@ also show overall period profit & loss."))
                        (gnc:gnc-monetary-commodity monetary)
                        common-currency))
                  (has-price? (gnc:gnc-monetary-commodity monetary))
-                 (exchange-fn
-                  monetary common-currency
-                  (cond
-                   ((eq? price-source 'pricedb-latest) (current-time))
-                   ((eq? col-idx 'overall-period) (last report-dates))
-                   ((eq? report-type 'balsheet) (list-ref report-dates col-idx))
-                   ((eq? report-type 'pnl) (list-ref report-dates (1+ col-idx))))))))
+                 (let ((col-date (col-idx->price-date col-idx)))
+                   (exchange-fn monetary common-currency col-date)))))
 
          ;; the following function generates an gnc:html-text object
          ;; to dump exchange rate for a particular column. From the
@@ -827,14 +782,15 @@ also show overall period profit & loss."))
                (lambda (commodity)
                  (let ((orig-monetary (gnc:make-gnc-monetary commodity 1)))
                    (if (has-price? commodity)
-                       (let* ((conv-monetary (convert-curr-fn orig-monetary col-idx))
-                              (conv-amount (gnc:gnc-monetary-amount conv-monetary)))
+                       (let ((price (gnc:case-price-fn
+                                     price-source common-currency
+                                     (col-idx->price-date col-idx))))
                          (gnc:html-text-append!
                           cell
                           (format #f "~a ~a"
                                   (gnc:monetary->string orig-monetary)
                                   (gnc:default-price-renderer common-currency
-                                                              conv-amount))))
+                                                              (price commodity)))))
                        (gnc:html-text-append!
                         cell
                         (string-append
@@ -894,10 +850,9 @@ also show overall period profit & loss."))
       (let ((balances
              (fold (lambda (a b) (if (member (car a) accts) (cons (cdr a) b) b))
                    '() alist)))
-        (list->vector
-         (if (null? balances)
-             (map (const (adder)) report-dates)
-             (apply map adder balances)))))
+        (if (null? balances)
+            (make-vector num-report-dates (adder))
+            (list->vector (apply map adder balances)))))
 
     (gnc:html-document-set-title!
      doc (with-output-to-string
@@ -929,11 +884,10 @@ also show overall period profit & loss."))
              ;; split is the last one at date boundary
              (accounts-splits-dates
               (map
-               (lambda (acc)
-                 (cons acc (let ((cols-data (assoc-ref accounts-cols-data acc)))
-                             (list->vector
-                              (map col-datum-get-last-split cols-data)))))
-               accounts))
+               (match-lambda
+                 ((acc . cols-data)
+                  (cons acc (list->vector (map col-datum-get-last-split cols-data)))))
+               accounts-cols-data))
 
              (get-cell-anchor-fn
               (lambda (account col-idx)
@@ -959,10 +913,10 @@ also show overall period profit & loss."))
              ;; dates. split-value-balance determined by transaction currency.
              (accounts-value-balances
               (map
-               (lambda (acc)
-                 (cons acc (let ((cols-data (assoc-ref accounts-cols-data acc)))
-                             (map col-datum-get-split-value-balance cols-data))))
-               accounts))
+               (match-lambda
+                 ((acc . cols-data)
+                  (cons acc (map col-datum-get-split-value-balance cols-data))))
+               accounts-cols-data))
 
              ;; a vector of collectors whereby each collector is the sum
              ;; of asset and liability split-value-balances at report
@@ -992,7 +946,7 @@ also show overall period profit & loss."))
                 (and-let* (common-currency
                            (date (case price-source
                                    ((pricedb-latest) (current-time))
-                                   (else (list-ref report-dates col-idx))))
+                                   (else (vector-ref report-dates-vec col-idx))))
                            (asset-liability-balance
                             (vector-ref asset-liability-balances col-idx))
                            (asset-liability-basis
@@ -1009,7 +963,7 @@ also show overall period profit & loss."))
               (lambda (col-idx)
                 (let* ((date (case price-source
                                ((pricedb-latest) (current-time))
-                               (else (list-ref report-dates col-idx))))
+                               (else (vector-ref report-dates-vec col-idx))))
                        (income-expense-balance
                         (vector-ref income-expense-balances-with-closing col-idx)))
                   (if (and common-currency
@@ -1034,7 +988,7 @@ also show overall period profit & loss."))
 
              (get-col-header-fn
               (lambda (accounts col-idx)
-                (let* ((date (list-ref report-dates col-idx))
+                (let* ((date (vector-ref report-dates-vec col-idx))
                        (header (qof-print-date date))
                        (cell (gnc:make-html-table-cell/markup
                               "total-label-cell" header)))
@@ -1054,7 +1008,8 @@ also show overall period profit & loss."))
                              (add-multicolumn-acct-table
                               table title accounts
                               maxindent get-cell-monetary-fn
-                              (iota (length report-dates))
+                              (iota num-report-dates)
+                              #:reverse-cols? reverse-chrono?
                               #:omit-zb-bals? omit-zb-bals?
                               #:show-zb-accts? show-zb-accts?
                               #:disable-account-indent? disable-account-indent?
@@ -1127,7 +1082,7 @@ also show overall period profit & loss."))
 
         (if (and common-currency show-rates?)
             (add-to-table multicol-table-right (G_ "Exchange Rates")
-                          asset-liability
+                          (append asset-liability equity-accounts)
                           #:get-col-header-fn get-exchange-rates-fn
                           #:show-accounts? #f
                           #:show-total? #f))
@@ -1143,14 +1098,10 @@ also show overall period profit & loss."))
                                                   optname-include-overall-period))
              (col-idx->datepair
               (lambda (idx)
-                (cond
-                 ((eq? idx 'overall-period)
-                  (cons (car report-dates) (last report-dates)))
-                 ((= idx (- (length report-dates) 2))
-                  (cons (list-ref report-dates idx) (last report-dates)))
-                 (else
-                  (cons (list-ref report-dates idx)
-                        (decdate (list-ref report-dates (1+ idx)) DayDelta))))))
+                (cons (if (eq? idx 'overall-period)
+                          startdate
+                          (vector-ref report-dates-vec idx))
+                      (col-idx->price-date idx))))
 
              (col-idx->monetarypair (lambda (balancelist idx)
                                       (if (eq? idx 'overall-period)
@@ -1171,24 +1122,22 @@ also show overall period profit & loss."))
                 (let ((datepair (col-idx->datepair col-idx))
                       (show-orig? (and common-currency #t))
                       (curr (or common-currency book-main-currency))
-                      (delta (or (not (eq? incr 'disabled)) 'MonthDelta))
-                      (price (or price-source 'pricedb-nearest))
                       (accts (if (pair? account) account (list account))))
                   (gnc:make-report-anchor
                    trep-uuid report-obj
                    (list
                     (list "General" "Start Date" (cons 'absolute (car datepair)))
                     (list "General" "End Date" (cons 'absolute (cdr datepair)))
-                    (list "General" "Show original currency amount" show-orig?)
-                    (list "General" "Common Currency" common-currency)
-                    (list "General" "Report's currency" curr)
+                    (list "Currency" "Show original currency amount" show-orig?)
+                    (list "Currency" "Common Currency" common-currency)
+                    (list "Currency" "Report's currency" curr)
                     (list "Display" "Amount" 'double)
                     (list "Accounts" "Accounts" accts))))))
 
              (chart
               (and-let* (include-chart?
                          (curr (or common-currency book-main-currency))
-                         (delta (or (not (eq? incr 'disabled)) 'MonthDelta))
+                         (delta (if (eq? incr 'disabled) 'MonthDelta incr))
                          (price (or price-source 'pricedb-nearest)))
                 (gnc:make-report-anchor
                  pnl-barchart-uuid report-obj
@@ -1224,11 +1173,12 @@ also show overall period profit & loss."))
                               table title accounts
                               maxindent get-cell-monetary-fn
                               (append
-                               (iota (1- (length report-dates)))
+                               (iota (1- num-report-dates))
                                (if (and include-overall-period?
-                                        (> (length report-dates) 2))
+                                        (> num-report-dates 2))
                                    '(overall-period)
                                    '()))
+                              #:reverse-cols? reverse-chrono?
                               #:omit-zb-bals? omit-zb-bals?
                               #:show-zb-accts? show-zb-accts?
                               #:disable-account-indent? disable-account-indent?
@@ -1295,9 +1245,6 @@ also show overall period profit & loss."))
       (gnc:html-document-add-object!
        doc multicol-table))
 
-    (gnc:html-document-add-object!
-     doc FOOTER-TEXT)
-
     (gnc:report-finished)
     ;; (gnc:html-document-set-style-text!
     ;;  doc " table, td{ border-width: 1px; border-style:solid; border-color: lightgray; border-collapse: collapse}")
@@ -1310,7 +1257,7 @@ also show overall period profit & loss."))
  'version 1
  'name balsheet-reportname
  'report-guid "065d5d5a77ba11e8b31e83ada73c5eea"
- 'menu-path (list gnc:menuname-experimental)
+ 'menu-path (list gnc:menuname-asset-liability)
  'options-generator (lambda () (multicol-report-options-generator 'balsheet))
  'renderer (lambda (rpt) (multicol-report-renderer rpt 'balsheet)))
 
@@ -1318,7 +1265,7 @@ also show overall period profit & loss."))
  'version 1
  'name pnl-reportname
  'report-guid "0e94fd0277ba11e8825d43e27232c9d4"
- 'menu-path (list gnc:menuname-experimental)
+ 'menu-path (list gnc:menuname-income-expense)
  'options-generator (lambda () (multicol-report-options-generator 'pnl))
  'renderer (lambda (rpt) (multicol-report-renderer rpt 'pnl)))
 
