@@ -10,8 +10,6 @@
 (use-modules (tests srfi64-extras))
 (use-modules (sxml simple))
 (use-modules (sxml xpath))
-(use-modules (system vm coverage))
-(use-modules (system vm vm))
 
 ;; Guide to the test-transaction.scm
 
@@ -28,16 +26,6 @@
 ;; (options->sxml) which in turn generates the transaction report, and
 ;; dumps the output at /tmp/test-trep-*.html for review.
 
-;; For coverage analysis, please amend (run-test) (if #f ...) to (if
-;; #t ...)  and this will run (coverage-test) instead, which will
-;; generate the coverage report in /tmp/lcov.info -- the latter can be
-;; converted to an html report using genhtml from
-;; http://ltp.sourceforge.net/coverage/lcov.php
-
-;; Please note the with-code-coverage has changed guile-2.0 to 2.2
-;; which does not require specifying the VM; I have no experience
-;; running in guile 2.2 and cannot confirm syntax.
-
 ;; copied from transaction.scm
 (define trep-uuid "2fe3b9833af044abb929a88d5a59620f")
 (define reconcile-uuid "e45218c6d76f11e7b5ef0800277ef320")
@@ -46,23 +34,6 @@
 (setlocale LC_ALL "C")
 
 (define (run-test)
-  (if #f
-      (coverage-test)
-      (run-test-proper)))
-
-(define (coverage-test)
-  (let* ((currfile (dirname (current-filename)))
-         (path (string-take currfile (string-rindex currfile #\/))))
-    (add-to-load-path path))
-  (call-with-values
-      (lambda()
-        (with-code-coverage run-test-proper))
-    (lambda (data result)
-      (let ((port (open-output-file "/tmp/lcov.info")))
-        (coverage-data->lcov data port)
-        (close port)))))
-
-(define (run-test-proper)
   (test-runner-factory gnc:test-runner)
   (test-begin "transaction.scm")
   (null-test)
@@ -170,6 +141,10 @@
         (set-option! options "Display" "Amount" 'single)
         options))
 
+    ;; to test Sorting/Show Account Description
+    (xaccAccountSetDescription income "Salaries etc")
+    (xaccAccountSetCode expense "EXP-001")
+
     ;; This will make all accounts use default currency (I think depends on locale)
     (for-each
      (lambda(pair)
@@ -257,7 +232,7 @@
     ;;                     (xaccSplitGetAmount s)
     ;;                     (xaccAccountGetName (xaccSplitGetAccount (xaccSplitGetOtherSplit s)))
     ;;                     ))
-    ;;           (xaccAccountGetSplitList bank))
+    ;;           (xaccAccountGetSplits bank))
 
     ;; Finally we can begin testing
     (test-begin "general options")
@@ -299,7 +274,16 @@
         (test-equal "month total present, and is $190 in original-currency"
           '("$190.00")
           (get-row-col sxml 6 -1))
-        ))
+        )
+
+      ;; dual columns with all running totals
+      (set-option! options "Display" "Running Totals" 'all)
+      (let ((sxml (options->sxml options "test column headers with original currency and running totals")))
+        (test-equal "default headers with all running totals, indented, includes common-currency"
+          '("Date" "Num" "Description" "Memo/Notes" "Account" "Amount (USD)"
+            "Running Secondary Subtotal (USD)" "Running Primary Subtotal (USD)" "Running Grand Total (USD)"
+            "Amount" "Running Secondary Subtotal" "Running Primary Subtotal" "Running Grand Total")
+          (get-row-col sxml 0 #f))))
 
     (test-end "general options")
 
@@ -478,7 +462,7 @@
          (set-option! options "Display" name #f))
        (list "Date" "Reconciled Date" "Num" "Description" "Memo" "Notes"
              "Account Name" "Other Account Name" "Shares" "Price" "Account Balance"
-             "Totals"))
+             "Grand Total"))
       (let ((sxml (options->sxml options "all columns off")))
         (test-assert "all display columns off, except amount and subtotals are enabled, there should be 2 columns"
           (= (length ((sxpath '(// (table 1) // (tr 1) // th)) sxml))
@@ -536,11 +520,12 @@
          (set-option! options "Display" name #t))
        (list "Date" "Reconciled Date" "Num" "Description" "Memo" "Notes"
              "Account Name" "Other Account Name" "Shares" "Price" "Account Balance"
-             "Totals" "Use Full Other Account Name" "Use Full Account Name"))
+             "Grand Total" "Use Full Other Account Name" "Use Full Account Name"))
+      (set-option! options "Display" "Running Totals" 'grand)
       (let* ((sxml (options->sxml options "all columns on")))
         (test-equal "all display columns on, displays correct columns"
           (list "Date" "Reconciled Date" "Num" "Description" "Memo/Notes" "Account"
-                "Transfer from/to" "Shares" "Price" "Amount" "Account Balance")
+                "Transfer from/to" "Shares" "Price" "Amount" "Account Balance" "Running Grand Total")
           (get-row-col sxml 0 #f))
         (test-assert "reconciled dates must be 01/03/70 or whitespace"
           (and-map
@@ -561,7 +546,7 @@
       (set-option! options "General" "Start Date" (cons 'absolute (gnc-dmy2time64 13 02 1971)))
       (set-option! options "General" "End Date" (cons 'absolute (gnc-dmy2time64 15 02 1971)))
       (set-option! options "Display" "Detail Level" 'multi-line)
-      (set-option! options "Display" "Totals" #f)
+      (set-option! options "Display" "Grand Total" #f)
       (set-option! options "Sorting" "Primary Subtotal" #f)
       (set-option! options "Sorting" "Primary Subtotal for Date Key" 'none)
       (set-option! options "Sorting" "Secondary Subtotal" #f)
@@ -700,6 +685,20 @@
         (test-equal "running balance, showing Balance b/f"
           (list "Bank: Balance b/f" "-$99.00")
           (get-row-col sxml 1 #f)))
+
+      ;; test running grand total
+      (set! options (default-testing-options))
+      (set-option! options "Display" "Running Totals" 'grand)
+      (set-option! options "Sorting" "Primary Subtotal" #f)
+      (let* ((sxml (options->sxml options "running grand total"))
+             (cs (char-set-adjoin! (char-set-copy char-set:digit) #\-)))
+        (test-equal "Last running grand total equals grand total"
+          (get-row-col sxml -3 -1)
+          (get-row-col sxml -1 -2))
+        (test-equal "Row 9 running grand total matches row 8 running grand total + row 9 amount"
+          (+ (string->number (string-filter cs (car (get-row-col sxml 8 -1))))
+             (string->number (string-filter cs (car (get-row-col sxml 9 -2)))))
+          (string->number (string-filter cs (car (get-row-col sxml 9 -1))))))
       )
 
     (test-end "display options")
@@ -711,7 +710,7 @@
       (set-option! options "General" "End Date" (cons 'absolute (gnc-dmy2time64 31 12 1970)))
       ;;(set-option! options "Accounts" "Accounts" (gnc-account-get-descendants (gnc-account-get-root bank)))
       (set-option! options "Accounts" "Accounts" (list bank))
-      (set-option! options "Display" "Totals" #f)
+      (set-option! options "Display" "Grand Total" #f)
       (set-option! options "Display" "Other Account Name" #t)
       (set-option! options "Filter" "Void Transactions" 'both)
       (set-option! options "Sorting" "Primary Subtotal" #f)
@@ -773,7 +772,7 @@
       (set-option! options "Sorting" "Primary Subtotal" #t)
       (set-option! options "Sorting" "Secondary Key" 'date)
       (set-option! options "Sorting" "Secondary Subtotal for Date Key" 'monthly)
-      (set-option! options "Display" "Totals" #t)
+      (set-option! options "Display" "Grand Total" #t)
       (set-option! options "Sorting" "Secondary Subtotal for Date Key" 'quarterly)
       (set-option! options "Sorting" "Show subtotals only (hide transactional data)" #t)
       (let* ((sxml (options->sxml options "sorting=account-name, date-quarterly, subtotals only")))
@@ -783,7 +782,7 @@
 
       (set! options (default-testing-options))
       (set-option! options "Accounts" "Accounts" (gnc-account-get-descendants (gnc-account-get-root bank)))
-      (set-option! options "Display" "Totals" #t)
+      (set-option! options "Display" "Grand Total" #t)
       (set-option! options "Display" "Amount" 'double)
       (set-option! options "Currency" "Show original currency amount" #t)
       (set-option! options "General" "Table for Exporting" #f)
@@ -793,7 +792,8 @@
       (set-option! options "Sorting" "Secondary Key" 'date)
       (set-option! options "Sorting" "Secondary Subtotal for Date Key" 'quarterly)
       (set-option! options "Sorting" "Show Informal Debit/Credit Headers" #t)
-      (set-option! options "Sorting" "Show Account Description" #t)
+      (set-option! options "Sorting" "Show Account Description" #f)
+      (set-option! options "Sorting" "Show Account Code" #f)
       (let* ((sxml (options->sxml options "sorting=date, friendly headers")))
         (test-equal "expense acc friendly headers"
           '("Expenses" "Expense" "Rebate")
@@ -802,8 +802,18 @@
           '("Income" "Charge" "Income")
           (get-row-col sxml 91 #f)))
 
+      (set-option! options "Sorting" "Show Account Description" #t)
+      (set-option! options "Sorting" "Show Account Code" #t)
+      (let* ((sxml (options->sxml options "sorting=date, friendly headers with acct desc/code")))
+        (test-equal "expense acc friendly headers"
+          '("EXP-001 Expenses" "Expense" "Rebate")
+          (get-row-col sxml 69 #f))
+        (test-equal "income acc friendly headers"
+          '("Income: Salaries etc" "Charge" "Income")
+          (get-row-col sxml 91 #f)))
+
       (set-option! options "Accounts" "Accounts" (list bank))
-      (set-option! options "Display" "Totals" #f)
+      (set-option! options "Display" "Grand Total" #f)
       (set-option! options "Sorting" "Show subtotals only (hide transactional data)" #t)
       (let* ((sxml (options->sxml options "sorting=date quarterly")))
         (test-equal "quarterly subtotals are correct"
@@ -840,7 +850,80 @@
           (get-row-col sxml #f 4))
         (test-equal "weekly subtotals are correct (2)"
           '("$15.00" "$8.00" "$100.00")
-          (get-row-col sxml #f 5))))
+          (get-row-col sxml #f 5)))
+
+      ;; test running subtotals
+      (set! options (default-testing-options))
+      (set-option! options "Display" "Grand Total" #f)
+      (set-option! options "Display" "Running Totals" 'sub)
+      (set-option! options "Sorting" "Primary Key" 'account-name)
+      (set-option! options "Sorting" "Primary Subtotal" #t)
+      (set-option! options "Sorting" "Secondary Key" 'date)
+      (set-option! options "Sorting" "Secondary Subtotal for Date Key" 'monthly)
+      (let* ((sxml (options->sxml options "running subtotals"))
+             (cs (char-set-adjoin! (char-set-copy char-set:digit) #\-)))
+        (test-equal "Running subtotal columns are present"
+          (list "Date" "Num" "Description" "Memo/Notes" "Account" "Amount"
+                "Running Secondary Subtotal" "Running Primary Subtotal")
+          (get-row-col sxml 0 #f))
+        (test-equal "Last running primary subtotal equals primary subtotal"
+          (get-row-col sxml -3 -1)
+          (get-row-col sxml -1 -3))
+        (test-equal "Row 10 running primary subtotal matches row 9 running primary subtotal + row 10 amount"
+          (+ (string->number (string-filter cs (car (get-row-col sxml 9 -1))))
+             (string->number (string-filter cs (car (get-row-col sxml 10 -3)))))
+          (string->number (string-filter cs (car (get-row-col sxml 10 -1)))))
+        (test-equal "In first secondary group, last running secondary subtotal equals secondary subtotal"
+          (get-row-col sxml 5 -2)
+          (get-row-col sxml 6 -3))
+        (test-equal "Row 10 running secondary subtotal matches row 9 running secondary subtotal + row 10 amount"
+          (+ (string->number (string-filter cs (car (get-row-col sxml 9 -2))))
+             (string->number (string-filter cs (car (get-row-col sxml 10 -3)))))
+          (string->number (string-filter cs (car (get-row-col sxml 10 -2))))))
+      ;; test all running totals shown
+      (set-option! options "Display" "Running Totals" 'all)
+      (let* ((sxml (options->sxml options "all running totals"))
+             (cs (char-set-adjoin! (char-set-copy char-set:digit) #\-)))
+        (test-equal "All running total columns are present"
+          (list "Date" "Num" "Description" "Memo/Notes" "Account" "Amount"
+                "Running Secondary Subtotal" "Running Primary Subtotal" "Running Grand Total")
+          (get-row-col sxml 0 #f)))
+      ;; test no running totals shown
+      (set-option! options "Display" "Running Totals" 'none)
+      (let* ((sxml (options->sxml options "no running totals"))
+             (cs (char-set-adjoin! (char-set-copy char-set:digit) #\-)))
+        (test-equal "No running total columns are present"
+          (list "Date" "Num" "Description" "Memo/Notes" "Account" "Amount")
+          (get-row-col sxml 0 #f)))
+
+      ;; test that only columns with subtotals are displayed when
+      ;; "Show subtotals only (hide transactional data)" is selected
+      (set! options (default-testing-options))
+      (for-each
+      (lambda (name)
+        (set-option! options "Display" name #t))
+      (list "Date" "Reconciled Date" "Num" "Description" "Memo" "Notes"
+            "Account Name" "Other Account Name" "Shares" "Price" "Account Balance"
+            "Grand Total" "Use Full Other Account Name" "Use Full Account Name"))
+      (set-option! options "Display" "Running Totals" 'all)
+      (set-option! options "Sorting" "Primary Key" 'account-name)
+      (set-option! options "Sorting" "Primary Subtotal" #t)
+      (set-option! options "Sorting" "Secondary Key" 'date)
+      (set-option! options "Sorting" "Secondary Subtotal for Date Key" 'monthly)
+      (set-option! options "Sorting" "Show subtotals only (hide transactional data)" #t)
+      (set-option! options "Currency" "Common Currency" #t)
+      (set-option! options "Currency" "Show original currency amount" #t)
+      (let* ((sxml (options->sxml options "show subtotals only, single column")))
+        (test-equal "all display columns on with single amount; show subtotals only, so only amount columns are shown"
+          (list "Amount (USD)" "Amount")
+          (get-row-col sxml 0 #f)))
+
+      (set-option! options "Display" "Amount" 'double)
+      (let* ((sxml (options->sxml options "show subtotals only, dual column")))
+        (test-equal "all display columns on with dual amount; show subtotals only, so only debit/credit columns are shown"
+          (list "Debit (USD)" "Credit (USD)" "Debit" "Credit")
+          (get-row-col sxml 0 #f)))
+    )
 
     (test-end "sorting options")
 
@@ -937,8 +1020,8 @@
       (set-option! options "Sorting" "Secondary Subtotal for Date Key" 'monthly)
 
       (let* ((template (gnc:find-report-template trep-uuid))
-             (constructor (record-constructor <report>))
-             (report (constructor trep-uuid "bar" options #t #t #f #f ""))
+             (report-id (gnc:make-report trep-uuid options))
+             (report (gnc-report-find report-id))
              (renderer (gnc:report-template-renderer template))
              (document (renderer report #:export-type 'csv)))
         (test-assert "csv output has no export error"
