@@ -117,6 +117,7 @@ setup_memory (Fixture* fixture, gconstpointer pData)
     Split* spl1, *spl2;
     gnc_commodity_table* table;
     gnc_commodity* currency;
+    Time64 now{gnc_time(nullptr)};
 
     gnc_module_init_backend_dbi();
     root = gnc_book_get_root_account (book);
@@ -131,10 +132,10 @@ setup_memory (Fixture* fixture, gconstpointer pData)
     xaccAccountSetCommodity (acct1, currency);
 
     auto frame = qof_instance_get_slots (QOF_INSTANCE (acct1));
-    frame->set ({"int64-val"}, new KvpValue (INT64_C (100)));
+    frame->set ({"int64-val"}, new KvpValue (INT64_C (2148634028)));
     frame->set ({"double-val"}, new KvpValue (3.14159));
     frame->set ({"numeric-val"}, new KvpValue (gnc_numeric_zero ()));
-    frame->set ({"time-val"}, new KvpValue (gnc_time(nullptr)));
+    frame->set ({"time-val"}, new KvpValue (now));
     frame->set ({"string-val"}, new KvpValue (g_strdup ("abcdefghijklmnop")));
     auto guid = qof_instance_get_guid (QOF_INSTANCE (acct1));
     frame->set ({"guid-val"}, new KvpValue (const_cast<GncGUID*> (guid_copy (
@@ -669,6 +670,8 @@ test_suite_gnc_backend_dbi (void)
     mysql_url.append(getenv("TEST_MYSQL_URL") ? getenv("TEST_MYSQL_URL") : "");
     pgsql_url.append(getenv("TEST_PGSQL_URL") ? getenv("TEST_PGSQL_URL") : "");
 
+    sort(drivers.begin(), drivers.end());
+
     for (auto name : drivers)
     {
         if (name == "sqlite3")
