@@ -66,8 +66,8 @@
 #include "gnc-locale-utils.hpp"
 #include <boost/filesystem.hpp>
 #include <boost/locale.hpp>
+#include <regex>
 #include <iostream>
-#include <numeric>
 
 /* Below cvt and bfs_locale should be used with boost::filesystem::path (bfs)
  * objects created alter in this source file. The rationale is as follows:
@@ -210,7 +210,10 @@ gchar *gnc_file_path_relative_part (const gchar *prefix, const gchar *path)
 {
     std::string p{path};
     if (p.find(prefix) == 0)
-        return g_strdup(p.substr(strlen(prefix)).c_str());
+    {
+        auto str = p.substr(strlen(prefix));
+        return g_strdup(str.c_str());
+    }
     return g_strdup(path);
 }
 
@@ -1303,27 +1306,44 @@ gnc_filepath_locate_doc_file (const gchar *name)
     return result;
 }
 
-GList *
-gnc_list_all_paths (void)
+std::vector<EnvPaths>
+gnc_list_all_paths ()
 {
     if (gnc_userdata_home.empty())
         gnc_filepath_init ();
 
-    std::vector<EnvPaths> paths
-        { { "GNC_USERDATA_DIR", gnc_userdata_home_str.c_str(), true},
-          { "GNC_USERCONFIG_DIR", gnc_userconfig_home_str.c_str(), true },
-          { "GNC_BIN", g_getenv ("GNC_BIN"), false },
-          { "GNC_LIB", g_getenv ("GNC_LIB"), false },
-          { "GNC_CONF", g_getenv ("GNC_CONF"), false },
-          { "GNC_DATA", g_getenv ("GNC_DATA"), false },
-        };
-    auto accum = [](const auto& a, const auto& b)
-    {
-        EnvPaths *ep = g_new0 (EnvPaths, 1);
-        *ep = b;
-        return g_list_prepend (a, ep);
+    return {
+        { "GNC_USERDATA_DIR", gnc_userdata_home_str.c_str(), true},
+        { "GNC_USERCONFIG_DIR", gnc_userconfig_home_str.c_str(), true },
+        { "GNC_BIN", g_getenv ("GNC_BIN"), false },
+        { "GNC_LIB", g_getenv ("GNC_LIB"), false },
+        { "GNC_CONF", g_getenv ("GNC_CONF"), false },
+        { "GNC_DATA", g_getenv ("GNC_DATA"), false },
     };
-    return std::accumulate (paths.rbegin(), paths.rend(), (GList*) nullptr, accum);
 }
 
+static const std::regex
+backup_regex (".*[.](?:xac|gnucash)[.][0-9]{14}[.](?:xac|gnucash)$");
+
+gboolean gnc_filename_is_backup (const char *filename)
+{
+    return std::regex_match (filename, backup_regex);
+}
+
+static const std::regex
+datafile_regex (".*[.](?:xac|gnucash)$");
+
+gboolean gnc_filename_is_datafile (const char *filename)
+{
+    return !gnc_filename_is_backup (filename) &&
+        std::regex_match (filename, datafile_regex);
+}
+
+std::ofstream
+gnc_open_filestream(const char* path)
+{
+    bfs::path bfs_path(path, cvt);
+    bfs_path.imbue(bfs_locale);
+    return std::ofstream(bfs_path.c_str());
+}
 /* =============================== END OF FILE ========================== */
