@@ -188,54 +188,61 @@ static QuoteSourceList currency_quote_sources =
  */
 static QuoteSourceList single_quote_sources =
 {
-    { false, SOURCE_SINGLE, "Alphavantage, US", "alphavantage" },
-    { false, SOURCE_SINGLE, "Amsterdam Euronext eXchange, NL", "aex" },
-    { false, SOURCE_SINGLE, "Association of Mutual Funds in India", "amfiindia" },
-    { false, SOURCE_SINGLE, "Australian Stock Exchange, AU", "asx" },
-    { false, SOURCE_SINGLE, "Canada Mutual", "canadamutual" },
-    { false, SOURCE_SINGLE, "Deka Investments, DE", "deka" },
-    { false, SOURCE_SINGLE, "Dutch", "dutch" },
-    { false, SOURCE_SINGLE, "DWS, DE", "dwsfunds" },
-    { false, SOURCE_SINGLE, "Financial Times Funds service, GB", "ftfunds" },
-    { false, SOURCE_SINGLE, "Finanzpartner, DE", "finanzpartner" },
-    { false, SOURCE_SINGLE, "GoldMoney spot rates, JE", "goldmoney" },
-    { false, SOURCE_SINGLE, "Google Web, US Stocks", "googleweb" },
-    { false, SOURCE_SINGLE, "India Mutual", "indiamutual" },
-    { false, SOURCE_SINGLE, "Morningstar, GB", "morningstaruk" },
-    { false, SOURCE_SINGLE, "Morningstar, JP", "morningstarjp" },
-    { false, SOURCE_SINGLE, "New Zealand stock eXchange, NZ", "nzx" },
-    { false, SOURCE_SINGLE, "Paris Stock Exchange/Boursorama, FR", "bourso" },
-    { false, SOURCE_SINGLE, "Romania", "romania" },
-    { false, SOURCE_SINGLE, "SIX Swiss Exchange shares, CH", "six" },
-    { false, SOURCE_SINGLE, "Skandinaviska Enskilda Banken, SE", "seb_funds" },
-    { false, SOURCE_SINGLE, "Sharenet, ZA", "za" },
-    { false, SOURCE_SINGLE, "TIAA-CREF, US", "tiaacref" },
-    { false, SOURCE_SINGLE, "Toronto Stock eXchange, CA", "tsx" },
-    { false, SOURCE_SINGLE, "T. Rowe Price", "troweprice" },
-    { false, SOURCE_SINGLE, "T. Rowe Price, US", "troweprice_direct" },
-    { false, SOURCE_SINGLE, "Union Investment, DE", "unionfunds" },
-    { false, SOURCE_SINGLE, "US Govt. Thrift Savings Plan", "tsp" },
-    { false, SOURCE_SINGLE, "Yahoo as JSON", "yahoo_json" },
-    { false, SOURCE_SINGLE, "Yahoo Web", "yahooweb" },
-    { false, SOURCE_SINGLE, "YH Finance (FinanceAPI)", "financeapi" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Alphavantage"), "alphavantage" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Amsterdam Euronext eXchange, NL"), "aex" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Association of Mutual Funds in India"), "amfiindia" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Athens Exchange Group, GR"), "asegr" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Australian Stock Exchange, AU"), "asx" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Italian Stock Exchange, IT"), "borsa_italiana" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "BSE India, IN"), "bseindia" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Bucharest Stock Exchange, RO"), "bvb" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "China Merchants Bank, CN"), "cmbchina" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Colombo Stock Exchange, LK"), "cse" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "comdirect, DE"), "comdirect" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Consors Bank, DE"), "consorsbank" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Deka Investments, DE"), "deka" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Financial Times Funds service, GB"), "ftfunds" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Finanzpartner, DE"), "finanzpartner" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "FondsWeb, DE"), "fondsweb" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "GoldMoney precious metals"), "goldmoney" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Google Web, US Stocks"), "googleweb" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Market Watch"), "marketwatch" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Morningstar, CH"), "morningstarch" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Morningstar, GB"), "morningstaruk" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Morningstar, JP"), "morningstarjp" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Motley Fool"), "fool" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "New Zealand stock eXchange, NZ"), "nzx" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "NSE (National Stock Exchange), IN"), "nseindia" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "OnVista, DE"), "onvista"},
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Paris Stock Exchange/Boursorama, FR"), "bourso" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "S-Investor, DE"), "sinvestor"},
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Sharenet, ZA"), "za" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "SIX Swiss Exchange shares, CH"), "six" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "StockData"), "stockdata" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Stooq, PL"), "stooq" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Tesouro Direto bonds, BR"), "tesouro_direto" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Toronto Stock eXchange, CA"), "tsx" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Tradegate, DE"), "tradegate" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Treasury Direct bonds, US"), "treasurydirect" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Twelve Data"), "twelvedata" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Union Investment, DE"), "unionfunds" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "US Govt. Thrift Savings Plan"), "tsp" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "XETRA, DE"), "xetra" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Yahoo as JSON"), "yahoo_json" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "Yahoo Web"), "yahooweb" },
+    { false, SOURCE_SINGLE, NC_("FQ Source", "YH Finance (FinanceAPI)"), "financeapi" },
 };
 
+// Finance::Quote defines these as failover methods
 static QuoteSourceList multiple_quote_sources =
 {
-    { false, SOURCE_MULTI, "Australia (ASX, ...)", "australia" },
-    { false, SOURCE_MULTI, "Canada (Alphavantage, TSX, ...)", "canada" },
-    { false, SOURCE_MULTI, "Canada Mutual (Fund Library, StockHouse, ...)", "canadamutual" },
-    { false, SOURCE_MULTI, "Dutch (AEX, ...)", "dutch" },
-    { false, SOURCE_MULTI, "Europe (asegr,.bsero, hex ...)", "europe" },
-    { false, SOURCE_MULTI, "India Mutual (AMFI, ...)", "indiamutual" },
-    { false, SOURCE_MULTI, "France (bourso, ĺerevenu, ...)", "france" },
-    { false, SOURCE_MULTI, "Nasdaq (alphavantage, yahoo_json, ...)", "nasdaq" },
-    { false, SOURCE_MULTI, "NYSE (alphavantage, yahoo_json, ...)", "nyse" },
-    { false, SOURCE_MULTI, "South Africa (Sharenet, ...)", "za" },
-    { false, SOURCE_MULTI, "Romania (BSE-RO, ...)", "romania" },
-    { false, SOURCE_MULTI, "T. Rowe Price", "troweprice" },
-    { false, SOURCE_MULTI, "U.K. Funds (citywire, FTfunds, MStar, tnetuk, ...)", "ukfunds" },
-    { false, SOURCE_MULTI, "USA (alphavantage, yahoo_json, ...)", "usa" },
+    { false, SOURCE_MULTI, NC_("FQ Source", "Canada (Alphavantage, TMX)"), "canada" },
+    { false, SOURCE_MULTI, NC_("FQ Source", "Europe (ASEGR, Bourso, …)"), "europe" },
+    { false, SOURCE_MULTI, NC_("FQ Source", "India (BSEIndia, NSEIndia)"), "india"},
+    { false, SOURCE_MULTI, NC_("FQ Source", "Nasdaq (Alphavantage, FinanceAPI, …)"), "nasdaq" },
+    { false, SOURCE_MULTI, NC_("FQ Source", "NYSE (Alphavantage, FinanceAPI, …)"), "nyse" },
+    { false, SOURCE_MULTI, NC_("FQ Source", "U.K. Funds (FTfunds, MorningstarUK)"), "ukfunds" },
+    { false, SOURCE_MULTI, NC_("FQ Source", "USA (Alphavantage, FinanceAPI, …)"), "usa" },
 };
 
 static QuoteSourceList new_quote_sources;
