@@ -23,7 +23,16 @@
 \********************************************************************/
 
 #include "../gnc-datetime.hpp"
+#include "../gnc-date.h"
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wcpp"
 #include <gtest/gtest.h>
+#pragma GCC diagnostic pop
+
+/* Backdoor to enable unittests to temporarily override the timezone: */
+class TimeZoneProvider;
+void _set_tzp(TimeZoneProvider& tz);
+void _reset_tzp();
 
 TEST(gnc_date_constructors, test_default_constructor)
 {
@@ -83,6 +92,12 @@ TEST(gnc_date_constructors, test_str_format_constructor)
         { "y-m-d",  "1985.3.12", 1985,  3, 12},
         { "y-m-d",      "3'6'8", 2003,  6,  8},
         { "y-m-d",   "20130801", 2013,  8,  1},
+        { "y-m-d", "2013 Aug 1", 2013,  8,  1},
+        { "y-m-d", "2013 Aug 01",2013,  8,  1},
+        { "y-m-d", "2013 August 01",    2013,  8,  1},
+        { "y-m-d", "2013-August-1",     2013,  8,  1},
+        { "y-m-d", "2009/Nov/04",2009, 11,  4},
+        { "y-m-d","1985.Mar.12", 1985,  3, 12},
         { "d-m-y", "01-08-2013", 2013,  8,  1},
         { "d-m-y",  "01-8-2013", 2013,  8,  1},
         { "d-m-y",  "1-08-2013", 2013,  8,  1},
@@ -95,6 +110,9 @@ TEST(gnc_date_constructors, test_str_format_constructor)
         { "d-m-y",  "12.3.1985", 1985,  3, 12},
         { "d-m-y",      "8'6'3", 2003,  6,  8},
         { "d-m-y",   "01082013", 2013,  8,  1},
+        { "d-m-y", "1 Aug 2013", 2013,  8,  1},
+        { "d-m-y", "1 Sep 2013", 2013,  9,  1},
+        { "d-m-y", "1 September 2013",  2013,  9,  1},
         { "m-d-y", "08-01-2013", 2013,  8,  1},
         { "m-d-y",  "8-01-2013", 2013,  8,  1},
         { "m-d-y",  "08-1-2013", 2013,  8,  1},
@@ -107,6 +125,8 @@ TEST(gnc_date_constructors, test_str_format_constructor)
         { "m-d-y",  "3.12.1985", 1985,  3, 12},
         { "m-d-y",      "6'8'3", 2003,  6,  8},
         { "m-d-y",   "08012013", 2013,  8,  1},
+        { "m-d-y", "November 4, 2009",  2009, 11,  4},
+        { "m-d-y", "Nov 4, 2009",       2009, 11,  4},
         {   "d-m",      "01-08",   curr_year,  8,  1},
         {   "d-m",       "01-8",   curr_year,  8,  1},
         {   "d-m",       "1-08",   curr_year,  8,  1},
@@ -124,6 +144,29 @@ TEST(gnc_date_constructors, test_str_format_constructor)
         {   "m-d",        "6'8",   curr_year,  6,  8},
         {   "m-d",       "0801",   curr_year,  8,  1},
 
+        // invalid dates
+        { "d-m-y", "0 Aug 2013",          -1, -1, -1},
+        { "d-m-y", "31 Sep 2013",         -1, -1, -1},
+        { "d-m-y", "31 September 2013",   -1, -1, -1},
+        { "d-m-y", "31/11/2009",          -2, -2, -2},
+        { "d-m-y",  "34.3.1985",          -2, -2, -2},
+        { "m-d-y", "November 41, 2009",   -1, -1, -1},
+        { "m-d-y", "Nov 31, 2009",        -1, -1, -1},
+        { "y-m-d", "2013 Aug 0",          -1, -1, -1},
+        { "y-m-d", "2013 Feb 30",         -1, -1, -1},
+        { "y-m-d", "2013 August 0",       -1, -1, -1},
+        { "y-m-d", "2013-June-31",        -1, -1, -1},
+        { "y-m-d", "2009/Nov/0",          -1, -1, -1},
+        { "y-m-d",  "1985.Mar.32",        -1, -1, -1},
+
+        // 2-digit dates are not parsable with months as words
+        { "d-m-y", "1 Sep 13",            -1, -1, -1},
+        { "d-m-y", "1 September 13",      -1, -1, -1},
+        { "m-d-y", "November 4, 24",      -1, -1, -1},
+        { "m-d-y", "Nov 4, 23",           -1, -1, -1},
+        { "m-d-y", "Nov 29, 24",          -1, -1, -1},
+        { "y-m-d", "13-June-11",          -1, -1, -1},
+
         // ambiguous date formats
         // current parser doesn't know how to disambiguate
         // and hence refuses to parse
@@ -180,6 +223,10 @@ TEST(gnc_date_constructors, test_str_format_constructor)
         {
             got_year = got_month = got_day = -1;
         }
+        catch (const std::out_of_range&)
+        {
+            got_year = got_month = got_day = -2;
+        }
 
         EXPECT_TRUE ((got_year  == test_dates[i].exp_year) &&
                      (got_month == test_dates[i].exp_month) &&
@@ -248,7 +295,7 @@ TEST(gnc_date_operators, test_move_assignment)
 TEST(gnc_datetime_constructors, test_default_constructor)
 {
     GncDateTime atime;
-    long time_now = time(nullptr);
+    time64 time_now = gnc_time(nullptr);
     EXPECT_EQ(static_cast<time64>(atime), static_cast<time64>(time_now));
 }
 
@@ -335,26 +382,157 @@ TEST(gnc_datetime_constructors, test_struct_tm_constructor)
 
 TEST(gnc_datetime_constructors, test_gncdate_start_constructor)
 {
-    const ymd aymd = { 2017, 04, 20 };
+    const gnc_ymd aymd = { 2017, 04, 20 };
     GncDateTime atime(GncDate(aymd.year, aymd.month, aymd.day), DayPart::start);
     //Skipping timezone information as this can't be controlled.
     EXPECT_EQ(atime.format("%d-%m-%Y %H:%M:%S"), "20-04-2017 00:00:00");
 }
 
+/* Putting this here is a bit weird but it includes
+ * boost/date_time/local_time/local_time.hpp and that redefines struct
+ * tm in a way that breaks initializing the tm in
+ * test_struct_tm_constructor on Linux.
+ */
+#include "../gnc-timezone.hpp"
+/* Summertime transitions have been a recurring problem. At the time of adding
+ * this test the GncDateEditor was refusing to set the date to 28 October 2018
+ * in the Europe/London timezone.
+ */
+TEST(gnc_datetime_constructors, test_gncdate_BST_transition)
+{
+    const gnc_ymd begins = {2018, 03, 25};
+    const gnc_ymd ends = {2018, 10, 28};
+#ifdef __MINGW32__
+    TimeZoneProvider tzp{"GMT Standard Time"};
+#else
+    TimeZoneProvider tzp("Europe/London");
+#endif
+    _set_tzp(tzp);
+    GncDateTime btime(GncDate(begins.year, begins.month, begins.day), DayPart::start);
+    GncDateTime etime(GncDate(ends.year, ends.month, ends.day), DayPart::start);
+    _reset_tzp();
+    EXPECT_EQ(btime.format("%d-%m-%Y %H:%M:%S"), "25-03-2018 00:00:00");
+    EXPECT_EQ(etime.format("%d-%m-%Y %H:%M:%S"), "28-10-2018 00:00:00");
+}
+
 TEST(gnc_datetime_constructors, test_gncdate_end_constructor)
 {
-    const ymd aymd = { 2046, 11, 06 };
+    const gnc_ymd aymd = { 2046, 11, 06 };
     GncDateTime atime(GncDate(aymd.year, aymd.month, aymd.day), DayPart::end);
     //Skipping timezone information as this can't be controlled.
     EXPECT_EQ(atime.format("%d-%m-%Y %H:%M:%S"), "06-11-2046 23:59:59");
 }
 
+static ::testing::AssertionResult
+test_offset(time64 start_time, int hour, int offset1, int offset2,
+            const char* zone)
+{
+    GncDateTime gdt{start_time + hour * 3600};
+    if ((hour < 2 && gdt.offset() == offset1) ||
+        (hour >= 2 && gdt.offset() == offset2))
+        return ::testing::AssertionSuccess();
+    else
+        return ::testing::AssertionFailure() << zone << ": " << gdt.format("%D %T %z %q") << " hour " << hour;
+}
+
+TEST(gnc_datetime_constructors, test_DST_start_transition_time)
+{
+#ifdef __MINGW32__
+    TimeZoneProvider tzp_can{"A.U.S Eastern Standard Time"};
+    TimeZoneProvider tzp_la{"Pacific Standard Time"};
+#else
+    TimeZoneProvider tzp_can("Australia/Canberra");
+    TimeZoneProvider tzp_la("America/Los_Angeles");
+#endif
+    _set_tzp(tzp_la);
+    for (auto hours = 0; hours < 23; ++hours)
+        EXPECT_TRUE(test_offset(1583657940, hours, -28800, -25200, "Los Angeles"));
+
+    _reset_tzp();
+    _set_tzp(tzp_can);
+    for (auto hours = 0; hours < 23; ++hours)
+        EXPECT_TRUE(test_offset(1601737140, hours, 36000, 39600, "Canberra"));
+    _reset_tzp();
+}
+
+TEST(gnc_datetime_constructors, test_DST_end_transition_time)
+{
+#ifdef __MINGW32__
+    TimeZoneProvider tzp_can{"A.U.S Eastern Standard Time"};
+    TimeZoneProvider tzp_la{"Pacific Standard Time"};
+#else
+    TimeZoneProvider tzp_can("Australia/Canberra");
+    TimeZoneProvider tzp_la("America/Los_Angeles");
+#endif
+    _set_tzp(tzp_la);
+    for (auto hours = 0; hours < 23; ++hours)
+        EXPECT_TRUE(test_offset(1604217540, hours, -25200, -28800, "Los Angeles"));
+    _reset_tzp();
+    _set_tzp(tzp_can);
+    for (auto hours = 0; hours < 23; ++hours)
+        EXPECT_TRUE(test_offset(1586008740, hours, 39600, 36000, "Canberra"));
+    _reset_tzp();
+}
+
+TEST(gnc_datetime_constructors, test_create_in_transition)
+{
+#ifdef __MINGW32__
+    TimeZoneProvider tzp_br{"E. South America Standard Time"};
+#else
+    TimeZoneProvider tzp_br("America/Sao_Paulo");
+#endif
+    _set_tzp(tzp_br);
+    /* Test Daylight Savings start: When Sao Paolo had daylight
+     * savings time it ended at 23:59:59 and the next second was
+     * 01:00:00 so that's when the day starts.
+     */
+    GncDate date0{"2018-11-03", "y-m-d"};
+    GncDateTime gncdt0{date0, DayPart::end};
+    EXPECT_EQ(gncdt0.format_zulu("%Y-%m-%d %H:%M:%S %Z"), "2018-11-04 02:59:59 UTC");
+    EXPECT_EQ(gncdt0.format("%Y-%m-%d %H:%M:%S %Z"), "2018-11-03 23:59:59 -03");
+    GncDate date1{"2018-11-04", "y-m-d"};
+    GncDateTime gncdt1{date1, DayPart::start};
+    EXPECT_EQ(gncdt1.format_zulu("%Y-%m-%d %H:%M:%S %Z"), "2018-11-04 03:00:00 UTC");
+    EXPECT_EQ(gncdt1.format("%Y-%m-%d %H:%M:%S %Z"), "2018-11-04 01:00:00 -02");
+    /* End of day, end of DST. We want one second before midnight in
+     * std time, i.e. -03. Unfortunately sometimes boost::date_time
+     * decides that it's still DST and returns the first one.
+     */
+    GncDate date2{"2018-02-17", "y-m-d"};
+    GncDateTime gncdt2{date2, DayPart::end};
+    if (gncdt2.offset() == -7200)
+    {
+        EXPECT_EQ(gncdt2.format_zulu("%Y-%m-%d %H:%M:%S %Z"), "2018-02-18 01:59:59 UTC");
+        EXPECT_EQ(gncdt2.format("%Y-%m-%d %H:%M:%S %Z"), "2018-02-17 23:59:59 -02");
+    }
+    else
+    {
+        EXPECT_EQ(gncdt2.format_zulu("%Y-%m-%d %H:%M:%S %Z"), "2018-02-18 02:59:59 UTC");
+        EXPECT_EQ(gncdt2.format("%Y-%m-%d %H:%M:%S %Z"), "2018-02-17 23:59:59 -03");
+    }
+   /* After February 2019 Sao Paulo discontinued Daylight
+     * Savings. This test checks to ensure that GncTimeZone doesn't
+     * try to project 2018's rule forward.
+     */
+    GncDate date3{"2019-11-01", "y-m-d"};
+    GncDateTime gncdt3{date3, DayPart::start};
+    EXPECT_EQ(gncdt3.format_zulu("%Y-%m-%d %H:%M:%S %Z"), "2019-11-01 03:00:00 UTC");
+    EXPECT_EQ(gncdt3.format("%Y-%m-%d %H:%M:%S %Z"), "2019-11-01 00:00:00 -03");
+}
+
 TEST(gnc_datetime_constructors, test_gncdate_neutral_constructor)
 {
-    const ymd aymd = { 2017, 04, 20 };
+#ifdef __MINGW32__
+    TimeZoneProvider tzp_la{"Pacific Standard Time"};
+#else
+    TimeZoneProvider tzp_la("America/Los_Angeles");
+#endif
+    _set_tzp(tzp_la);
+    const gnc_ymd aymd = { 2017, 04, 20 };
     GncDateTime atime(GncDate(aymd.year, aymd.month, aymd.day), DayPart::neutral);
-    GncDateTime gncdt(1492685940); /* 20 Apr 2017 10:59:00 Z */
-    /* The ymd constructor sets the time of day at 10:59:00 for
+    time64 date{1492685940};
+    GncDateTime gncdt(date); /* 20 Apr 2017 10:59:00 Z */
+    /* The gnc_ymd constructor sets the time of day at 10:59:00 for
      * timezones between UTC-10 and UTC+13. For other timezones the
      * time of day is adjusted to ensure a consistent date and the
      * adjustment invalidates the test, so skip it.
@@ -364,10 +542,49 @@ TEST(gnc_datetime_constructors, test_gncdate_neutral_constructor)
     if (gncdt.offset() >= max_western_offset &&
         gncdt.offset() <= max_eastern_offset)
     {
-        EXPECT_EQ(atime.format("%d-%m-%Y %H:%M:%S %z"), "20-04-2017 10:59:00 UTC");
+        EXPECT_EQ(atime.format_zulu("%d-%m-%Y %H:%M:%S %Z"),
+                  "20-04-2017 10:59:00 UTC");
+        EXPECT_EQ(date, static_cast<time64>(gncdt));
+        EXPECT_EQ(date, static_cast<time64>(atime));
     }
 }
 
+TEST(gnc_datetime_constructors, test_neutral_across_timezones)
+{
+    const gnc_ymd begins = {2018, 03, 05};
+    const gnc_ymd ends = {2018, 03, 15};
+    const time64 ten_days = 864000;
+#ifdef __MINGW32__
+    TimeZoneProvider tzp_lon{"GMT Standard Time"};
+    TimeZoneProvider tzp_perth{"W. Australia Standard Time"};
+    TimeZoneProvider tzp_la{"Pacific Standard Time"};
+#else
+    TimeZoneProvider tzp_lon("Europe/London");
+    TimeZoneProvider tzp_perth("Australia/Perth");
+    TimeZoneProvider tzp_la("America/Los_Angeles");
+#endif
+    _set_tzp(tzp_lon);
+    GncDateTime btime_lon(GncDate(begins.year, begins.month, begins.day), DayPart::neutral);
+    GncDateTime etime_lon(GncDate(ends.year, ends.month, ends.day), DayPart::neutral);
+    _reset_tzp();
+    _set_tzp(tzp_perth);
+    GncDateTime btime_perth(GncDate(begins.year, begins.month, begins.day), DayPart::neutral);
+    GncDateTime etime_perth(GncDate(ends.year, ends.month, ends.day), DayPart::neutral);
+    _reset_tzp();
+    _set_tzp(tzp_la);
+    GncDateTime btime_la(GncDate(begins.year, begins.month, begins.day), DayPart::neutral);
+    GncDateTime etime_la(GncDate(ends.year, ends.month, ends.day), DayPart::neutral);
+    _reset_tzp();
+
+    EXPECT_EQ(static_cast<time64>(btime_lon), static_cast<time64>(btime_perth));
+    EXPECT_EQ(static_cast<time64>(btime_lon), static_cast<time64>(btime_la));
+    EXPECT_EQ(static_cast<time64>(etime_lon), static_cast<time64>(etime_perth));
+    EXPECT_EQ(static_cast<time64>(etime_lon), static_cast<time64>(etime_la));
+    EXPECT_EQ(ten_days, static_cast<time64>(etime_lon) - static_cast<time64>(btime_lon));
+    EXPECT_EQ(ten_days, static_cast<time64>(etime_perth) - static_cast<time64>(btime_perth));
+    EXPECT_EQ(ten_days, static_cast<time64>(etime_la) - static_cast<time64>(btime_la));
+}
+
 TEST(gnc_datetime_functions, test_format)
 {
     GncDateTime atime(2394187200); //2045-11-13 12:00:00 Z
@@ -395,15 +612,34 @@ TEST(gnc_datetime_functions, test_date)
     EXPECT_EQ(ymd.day - (12 + atime.offset() / 3600) / 24, 13);
 }
 /* This test works only in the America/LosAngeles time zone and
- * there's no way at present to make it more flexible.
-TEST(gnc_datetime_functions, test_timezone_offset)
+ * there's no straightforward way to make it more flexible. It ensures
+ * that DST in that timezone transitions correctly for each day of the
+ * week in which March begins.
+
+ TEST(gnc_datetime_functions, test_timezone_offset)
 {
+    struct Timepair
+    {
+        time64 before;
+        time64 after;
+    };
+    std::array<Timepair, 7> years{
+        Timepair{1615633140, 1615719540}, //2021, Monday
+        Timepair{1457780340, 1457866740}, //2016, Tuesday
+        Timepair{1489229940, 1489316340}, //2017, Wednesday
+        Timepair{1520679540, 1520765940}, //2018, Thursday
+        Timepair{1552129140, 1552215540}, //2019, Friday
+        Timepair{1741431540, 1741517940}, //2025, Saturday
+        Timepair{1583578740, 1583665140}  //2020, Sunday
+    };
+    for (auto year : years)
+    {
+        GncDateTime before{year.before};
+        GncDateTime after{year.after};
+//        std::cerr << before.format_iso8601() << std::endl;
+        EXPECT_EQ(-28800, before.offset());
+        EXPECT_EQ(-25200, after.offset());
+    }
 
-    GncDateTime gncdt1(1488797940); //6 Mar 2017
-    EXPECT_EQ(-28800, gncdt1.offset());
-    GncDateTime gncdt2(1489661940);  //16 Mar 2017 10:59 Z
-    EXPECT_EQ(-25200, gncdt2.offset());
-    GncDateTime gncdt3(1490525940);  //26 Mar 2017
-    EXPECT_EQ(-25200, gncdt3.offset());
 }
 */
